/*
	NOM PAQUET: PKG_CONTROL
	
	FUNCIONALITAT: Permet ACTIVAR i DESACTIVAR tant el LOG com el mode TEST. Proporciona les operaciones necessriesper saber
				         si el LOG est activat o no, aix com tamb per registrar les operacions en el LOG.
*/

spool 4_1_Creacio_Pkg_Control.log;
  
create or replace
package pkg_control as

  -----------------
  -- PKG_CONTROL --
  -----------------

	-- Variable que cont el nom del paquet que s'utilitza per a registar-lo a la taula LOG_PROC_EXEC.
	pck_nom varchar2(11):='PKG_CONTROL';

	-- Variable que s'utilitza per determinar si el sistema es troba en mode TEST.
	v_test_actiu boolean;

	-- Funci que insereix a la taula LOG_PROC_EXEC, nicament si el sistema t activat el mode LOG, el nom del procediment i els parmetres amb els quals s'ha executat.
  -- La funci retorna un nou nmero de seqncia que ser amb el que es registrar el procediment a la taula LOG_PROC_EXEC.
  -- En el cas de que el sistema no tingui el mode LOG activat, la funci retorna un 0 i no registra l'execuci del procediment.
  function fun_reg_exec (
    p_proc varchar2,
    p_param varchar2
  ) return number;

	-- Si el id que es passa com a parmetre al procediment s major que 0, insereix a la taula LOG_PROC_HIST_EXEC el id del procediment, la data i hora en que s'executa i la paraula "INICI" al camp OPERACIO.
	procedure prc_exec_ini (
		p_id number
	);

	-- Si el id que es passa com a parmetre al procediment s major que 0, insereix a la taula LOG_PROC_HIST_EXEC el id del procediment, la data i hora en que s'executa i la paraula "FI" al camp OPERACIO.
	procedure prc_exec_fi (
		p_id number
	);

	-- Si el id que es passa com a parmetre al procediment s major que 0 i el sistema es troba en mode TEST, insereix a la taula LOG_PROC_HIST_EXEC el id del procediment, la data i hora en que s'executa i la paraula "TEST".
	procedure prc_exec_test (
    p_id number
	);

	-- Si el id que es passa com a parmetre al procediment s major que 0 i es produeix una excepci al executar el procs,
  -- insereix a la taula LOG_PROC_HIST_EXEC el id del procediment, la data i hora en que s'executa i la paraula "ERROR" al camp OPERACIO.
  procedure prc_exec_error (
		p_id number,
		p_error varchar2
	);

	-- Comprova si el mode LOG est activat al sistema. Si el parmetre LOG_ACTIU t el valor 'S' la funci retorna TRUE, si t el valor 'N' retorna FALSE.
  function fun_log_actiu return boolean;

	-- Comprova si el mode TEST est activat al sistema. Si el parmetre TEST_ACTIU t el valor 'S' la funci retorna TRUE, si t el valor 'N' retorna FALSE.
  function fun_test_actiu return boolean;

	-- Procediment que modifica el valor del parmetre LOG_ACTIU al valor 'S'.
	procedure prc_activa_log;

	-- Procediment que modifica el valor del parmetre LOG_ACTIU al valor 'N'.
	procedure prc_desactiva_log;

	-- Procediment que modifica el valor del parmetre TEST_ACTIU al valor 'S'.
	procedure prc_activa_test;

	-- Procediment que modifica el valor del parmetre TEST_ACTIU al valor 'N'.
	procedure prc_desactiva_test;

	-- Procediment que retorna el detall de les operacions realitzades per un id de log concret.
	procedure prc_log_id (p_id number);

	-- Procediment que retorna el detall de les execucions dels procediments realitzats entre les dates passades com a parmetres al cridar el procediment.
  procedure prc_log_data (p_data_inici date, p_data_fi date);

	-- Procediment que retorna les operacions que contenen el text que es passa com a parmetre quan es crida el procediment.
	procedure prc_cerca_log_text (p_text varchar2);

end pkg_control;
/

--------------------
-- COS DEL PAQUET --
--------------------
create or replace
package body pkg_control as

  ------------------
  -- FUN_REG_EXEC --
  ------------------
  function fun_reg_exec (
    p_proc varchar2,
    p_param varchar2
  ) return number is

    -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
    v_id number;
		-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
		v_log_actiu boolean;

  begin
		--  Assigna el valor que retorna la funci FUN_LOG_ACTIU a la variable V_LOG_ACTIU.
    v_log_actiu:=pkg_control.fun_log_actiu();
    if v_log_actiu then
      -- Si el sistema t el LOG activat s'assigna a la varibale V_ID el valor del segent nmero de la seqncia SEQ_LOG_PROC_EXEC.
			v_id:=seq_log_proc_exec.nextval;
      -- Insereix a la taula LOG_PROC_EXEC l'identificador del log, el nom del procediment i els parmetres amb els quals s'ha realitzat la seva crida.
			insert into log_proc_exec values(v_id,p_proc,p_param);
      -- Inserta a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "INICI" al camp OPERACIO.
      -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
			pkg_control.prc_exec_ini(v_id);
			commit;
			-- La funci retorna l'identificador del log registrat.
			return v_id;
		else
			-- Si el parmetre LOG_ACTIU t valor "N" la funci retorna 0.
			return 0;
		end if;
	end fun_reg_exec;


	------------------
	-- PRC_EXEC_INI --
	------------------
  procedure prc_exec_ini (
		p_id number
  ) is

	begin
		-- Si el id s major que 0 vol dir que el parmetre LOG_ACTIU est activat.
		if p_id > 0 then
      -- Si el parmetre LOG_ACTIU est activat insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i la paraula "INICI" al camp OPERACI.
			insert into log_proc_hist_exec values (p_id,sysdate,'INICI');
			commit;
		end if;
	end prc_exec_ini;


	-----------------
	-- PRC_EXEC_FI --
	-----------------
  procedure prc_exec_fi (
		p_id number
  ) is

  begin
		-- Si el id s major que 0 vol dir que el parmetre LOG_ACTIU est activat.
		if p_id > 0 then
      -- Si el parmetre LOG_ACTIU est activat insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i la paraula "FI" al camp OPERACI.
			insert into log_proc_hist_exec values (p_id,sysdate,'FI');
			commit;
		end if;
	end prc_exec_fi;


	-------------------
	-- PRC_EXEC_TEST --
	-------------------
	procedure prc_exec_test (
		p_id number
	) is

	begin
		-- Si el id s major que 0 vol dir que el parmetre LOG_ACTIU est activat.
		if p_id > 0 then
      -- Si el parmetre LOG_ACTIU est activat insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i la paraula "TEST" al camp OPERACI.
			insert into log_proc_hist_exec values (p_id,sysdate,'TEST');
			commit;
		end if;
	end prc_exec_test;


	--------------------
	-- PRC_EXEC_ERROR --
	--------------------
	procedure prc_exec_error (
    p_id number,
		p_error varchar2
  ) is

	begin
		-- Si el id s major que 0 vol dir que el parmetre LOG_ACTIU est activat.
		if p_id > 0 then
			-- Si el parmetre LOG_ACTIU est activat insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el missatge de l'error al camp OPERACI.
			insert into log_proc_hist_exec values (p_id,sysdate,p_error);
			commit;
		end if;
	end prc_exec_error;


	-------------------
	-- FUN_LOG_ACTIU --
	-------------------
	function fun_log_actiu return boolean is
		v_log_actiu varchar2(1);

  begin
		--  Assigna a la variable V_LOG_ACTIU el valor del parmetre de LOG_ACTIU de la taula CONFIG_SISTEMA.
		select valor into v_log_actiu from config_sistema where parametre ='LOG_ACTIU';
		if v_log_actiu = 'S' then
			-- Si el valor del parmetre s igual a "S" la funci retorna TRUE. Significa que la BD t el LOG activat.
			return true;
		else
			-- Si el valor del parmetre s igual a "N" la funci retorna FALSE. Significa que la BD t el LOG desactivat.
			return false;
		end if;
	end fun_log_actiu;


  --------------------
	-- FUN_TEST_ACTIU --
	--------------------
	function fun_test_actiu return boolean is
		v_test_actiu varchar2(1);

  begin
		--  Assigna a la variable V_TEST_ACTIU el valor del parmetre de TEST_ACTIU de la taula CONFIG_SISTEMA.
		select valor into v_test_actiu from config_sistema where parametre ='TEST_ACTIU';
		if v_test_actiu = 'S' then
			-- Si el valor del parmetre s igual a "S" la funci retorna TRUE. Significa que la BD t el TEST activat.
			return true;
		else
			-- Si el valor del parmetre s igual a "N" la funci retorna FALSE. Significa que la BD t el TEST desactivat.
			return false;
		end if;
	end fun_test_actiu;


	--------------------
	-- PRC_ACTIVA_LOG --
	--------------------
	procedure prc_activa_log is

  begin
		-- Actualitza el valor del parmetre LOG_ACTIU de la taula CONFIG_SISTEMA a "S".
		update config_sistema set valor = 'S' where parametre = 'LOG_ACTIU';
    commit;
	end prc_activa_log;


	-----------------------
	-- PRC_DESACTIVA_LOG --
	-----------------------
	procedure prc_desactiva_log is

  begin
		-- Actualitza el valor del parmetre LOG_ACTIU de la taula CONFIG_SISTEMA a "S".
		update config_sistema set valor = 'N' where parametre = 'LOG_ACTIU';
    commit;
	end prc_desactiva_log;


	---------------------
	-- PRC_ACTIVA_TEST --
	---------------------
	procedure prc_activa_test is

  begin
    -- Actualitza el valor del parmetre TEST_ACTIU de la taula CONFIG_SISTEMA a "S".
		update config_sistema set valor = 'S' where parametre = 'TEST_ACTIU';
		commit;
	end prc_activa_test;


	------------------------
	-- PRC_DESACTIVA_TEST --
	------------------------
	procedure prc_desactiva_test is

  begin
    -- Actualitza el valor del parmetre TEST_ACTIU de la taula CONFIG_SISTEMA a "N".
		update config_sistema set valor = 'N' where parametre = 'TEST_ACTIU';
		commit;
	end prc_desactiva_test;


	----------------
	-- PRC_LOG_ID --
	----------------
	procedure prc_log_id (
    p_id number
  ) is

  v_procediment varchar2(100);
  v_parametres varchar(100);
  v_id number;

  -- Cursor que s'utilitza per obtenir el nom i els parmetres del procediment amb el id de log igual al passat per parmetre.
  cursor c_proc is
    select procediment, parametres
      from log_proc_exec
     where id = p_id;
  -- Cursor que s'utilitza per obtenir totes les operacions i les dates i hores d'execuci del procediment, ordenades per data, amb el id de log igual al passat per parmetre.
  cursor c_proc_oper is
    select data, operacio
      from log_proc_hist_exec
     where id = p_id
     order by data asc;

  begin
	  -- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
		v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_LOG_ID','PAR1: '||p_id);
		-- Comprova si el sistema t el mode TEST actiu i emmagatzema el valor retornat a la variable V_TEST_ACTIU.
		v_test_actiu:=pkg_control.fun_test_actiu();
		if v_test_actiu then
      -- Si el sistema es troba en mode TEST es registra a la taula LOG_PROC_HIST_EXEC amb el text "TEST" al camp OPERACIO.
			pkg_control.prc_exec_test(p_id);
    end if;
    -- Obre el cursor c_proc.
	  open c_proc;
	  fetch c_proc into v_procediment, v_parametres;
	  -- Comprova si el cursor retorna registres.
	  if c_proc%found then
      dbms_output.put_line('++++++++++++++++++++++++++++++++++++++');
      dbms_output.put_line('ID DEL PROCEDIMENT: '||p_id);
      dbms_output.put_line('++++++++++++++++++++++++++++++++++++++');
      dbms_output.put_line('NOM DEL PROCEDIMENT: '||v_procediment);
  	  dbms_output.put_line('PARAMETRES D'' EXECUCI: '||v_parametres);
	  	for i in c_proc_oper loop
		   	dbms_output.put_line('DATA: '||i.data||'  OPERACIO: '||i.operacio);
      end loop;
    else
		 	dbms_output.put_line('++++++++++++++++++++++++++++++++++++++++++++++++++++');
  		dbms_output.put_line('NO EXISTEIXEN EXECUCIONS PEL LOG AMB EL ID: '||p_id);
	  	dbms_output.put_line('++++++++++++++++++++++++++++++++++++++++++++++++++++');
	  	dbms_output.new_line;
	  end if;
	  -- Tanca el cursor c_proc.
		close c_proc;
	  -- Inserta a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
	  pkg_control.prc_exec_fi(p_id);
  exception
	  -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
    -- nicament insereix el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
    when others then
      pkg_control.prc_exec_error(p_id,'ERROR: '||substr(sqlerrm,1,500));
  end prc_log_id;


	------------------
	-- PRC_LOG_DATA --
	------------------
  procedure prc_log_data (p_data_inici date, p_data_fi date) is
	-- Cursor que retorna els diferents id's dels logs corresponents als procediments que s'han executat entre les dues dates passades per parmetre.
	cursor c_id_proc is
    select distinct lphe.id
      from log_proc_exec lpe, log_proc_hist_exec lphe
     where lpe.id = lphe.id
       and lphe.data between p_data_inici and p_data_fi
     order by lphe.id asc;
  -- Varibale per emmagatzemar el valor del segent nmero de la seqncia SEQ_LOG_PROC_EXEC.
	v_id number;

	begin
    -- Regitra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
		v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_LOG_DATA','PAR1: '||p_data_inici||
                                                         ' - PAR2: '||p_data_fi);
	 	-- Comprova si el sistema t el mode TEST actiu i emmagatzema el valor retornat a la variable v_test_actiu.
	 	v_test_actiu:=pkg_control.fun_test_actiu();
	  if v_test_actiu then
	   	-- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC i es continua amb l'execuci.
		  pkg_control.prc_exec_test(v_id);
	  end if;
    for i in c_id_proc loop
      pkg_control.prc_log_id(i.id);
    end loop;
		-- Inserta a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		pkg_control.prc_exec_fi(v_id);
  exception
		-- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit i el detall del error.
    -- nicament insereix el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
	  when others then
      pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm, 1, 500));
  end prc_log_data;


	------------------------
	-- PRC_CERCA_LOG_TEXT --
	------------------------
  procedure prc_cerca_log_text (p_text varchar2) is
  -- Cursor que retorna els id's dels logs dels procediments que s'han executat i que contenen al camp operaci el text passat per parmetre.
	cursor c_id_proc is
	  select distinct id
      from log_proc_hist_exec
		 where operacio like '%'||p_text||'%';
	-- Varibale per emmagatzemar el valor del segent nmero de la seqncia SEQ_LOG_PROC_EXEC.
	v_id number;

  begin
    -- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_CERCA_LOG_TEXT','PAR1: '||p_text);
	  -- Comprova si el sistema t el mode TEST actiu i emmagatzema el valor retornat a la variable v_test_actiu.
	  v_test_actiu:=pkg_control.fun_test_actiu();
    if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC i es continua amb l'execuci.
			pkg_control.prc_exec_test(v_id);
    end if;
		for i in c_id_proc loop
			pkg_control.prc_log_id(i.id);
		end loop;
    -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
    pkg_control.prc_exec_fi(v_id);
  exception
		-- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit i del detall del error.
    -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
		when others then
		pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm, 1, 500));
  end prc_cerca_log_text;

end pkg_control;
/

spool off;
