/*
	NOM PAQUET: PKG_JUGADOR
	
	FUNCIONALITAT: Permet realitzar altes, baixes i modificacions de jugadors al sistema.

*/

spool 4_2_Creacio_Pkg_Jugador.log;
  
----------------
-- CAPAALERA --
----------------
create or replace
package pkg_jugador as

  -----------------
  -- PKG_JUGADOR --
  -----------------

  -- Variable que cont el nom del paquet que s'utilitza per a registar-lo a la taula LOG_PROC_EXEC.
	pck_nom varchar2(11):='PKG_JUGADOR';

	-- Variable que s'utilitza per determinar si el sistema es troba en mode TEST.
	v_test_actiu boolean;

	-- Procediment que s'utilitza per donar d'alta un nou jugador al sistema.
	procedure prc_jugador_alta (
		p_id             varchar2,
    p_nacionalitat   varchar2,
    p_federacio      varchar2,
    p_nom            varchar2,
    p_sexe           varchar2,  -- 1: Home i 2: Dona
    p_data_naixement date,
    p_alada         number,
    p_pes            number,
    p_posicio        number,  -- 1: Base, 2: Escorta, 3: Aler, 4: Aler-Pivot i 5: Pivot
    p_representant   varchar2,
    p_url_personal   varchar2,
    p_url_promocio   varchar2
	);

	-- Procediment que s'utilitza per donar de baixa el jugador amb id passat per parmetre.
	procedure prc_jugador_baixa (
		p_id varchar2
	);

	-- Procediment que s'utilitza per modificar les dades d'un jugador al sistema.
	procedure prc_jugador_modifica (
		p_id                 varchar2,
    p_nacionalitat       varchar2,
    p_federacio          varchar2,
    p_num_federat        number,
    p_nom                varchar2,
    p_sexe               varchar2,  -- 1: Home i 2: Dona
    p_data_naixement     date,
    p_alada             number,
    p_pes                number,
    p_posicio            number,  -- 1: Base, 2: Escorta, 3: Aler, 4: Aler-Pivot i 5: Pivot
    p_representant       varchar2,
    p_url_personal       varchar2,
    p_url_promocio       varchar2,
    p_ind_activitat      varchar2,  -- A: Actiu i R: Retirat
    p_estat              varchar2,  -- A: Alta i B: Baixa
    p_sub_estat          varchar2,  -- BM: Baixa mdica i MP: Motius personals
    p_data_disp_estimada date
	);

end pkg_jugador;
/

--------------------
-- COS DEL PAQUET --
--------------------
create or replace
package body pkg_jugador is

	----------------------
	-- PRC_JUGADOR_ALTA --
	----------------------
  procedure prc_jugador_alta (
	  p_id             varchar2,
    p_nacionalitat   varchar2,
    p_federacio      varchar2,
    p_nom            varchar2,
    p_sexe           varchar2,  -- 1: Home i 2: Dona
    p_data_naixement date,
    p_alada         number,
    p_pes            number,
    p_posicio        number,  -- 1: Base, 2: Escorta, 3: Aler, 4: Aler-Pivot i 5: Pivot
    p_representant   varchar2,
    p_url_personal   varchar2,
    p_url_promocio   varchar2
  ) is

  -- Variable que s'utilitza per emmagatzemar l'identificador nic del log que retorna la funci FUN_REG_EXEC.
  v_id number;
  r_jug jugadors%rowtype;

  begin
    -- Regitra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_JUGADOR_ALTA','PAR1: '||p_id||
                                                             ' - PAR2: '||p_nacionalitat||
                                                             ' - PAR3: '||p_federacio||
                                                             ' - PAR4: '||p_nom||
                                                             ' - PAR5: '||p_sexe||
                                                             ' - PAR6: '||p_data_naixement||
                                                             ' - PAR7: '||p_alada||
                                                             ' - PAR8: '||p_pes||
                                                             ' - PAR9: '||p_posicio||
                                                             ' - PAR10: '||p_url_personal||
                                                             ' - PAR11: '||p_url_promocio);
	  if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment per no s'insereix cap registre a la taula JUGADORS.
		pkg_control.prc_exec_test(v_id);
		else
      r_jug.id               :=   p_id;
      r_jug.nacionalitat     :=   p_nacionalitat;
      r_jug.federacio        :=   p_federacio;
      r_jug.num_federat      :=   seq_num_federat.nextval;
      r_jug.nom              :=   p_nom;
      r_jug.sexe             :=   p_sexe;
      r_jug.data_naixement   :=   p_data_naixement;
      r_jug.alada           :=   p_alada;
      r_jug.pes              :=   p_pes;
      r_jug.posicio          :=   p_posicio;
      r_jug.url_personal     :=   p_url_personal;
      r_jug.url_promocio     :=   p_url_promocio;
      r_jug.ind_activitat    :=   'A';
      r_jug.estat            :=   'A';
      -- Insereix a la taula JUGADORS el registre amb les dades introduides del nou jugador.
      insert into jugadors values r_jug;
     end if;
     -- Si s'ha introduit un representant en el procs d'alta s'insereix un registre a la taula JUGADORS_REPRESENTANT amb el id del jugador 
     -- i el del representant.
     if p_representant is not null then
       insert into jugadors_representants values (p_id,p_representant);
     end if;
     -- Confirma la inserci del nou registre a la taula JUGADORS.
     commit;
		 -- Inserta a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));
  end prc_jugador_alta;


	-----------------------
	-- PRC_JUGADOR_BAIXA --
	-----------------------
	procedure prc_jugador_baixa (
		p_id varchar2
	) is

  -- Variable que s'utilitza per emmagatzemar l'identificador nic del log que retorna la funci FUN_REG_EXEC.
  v_id number;

  cursor c_jug is
    select * from jugadors
     where id = p_id
       for update of id;
  i c_jug%rowtype;

  begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
		v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_JUGADOR_BAIXA','PAR1: '||p_id);
    if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment per no s'elimina cap registre de les taules: JUGADORS_EQUIPS, JUGADORS_REPRESENTANTS i JUGADORS.
			pkg_control.prc_exec_test(v_id);
		else
    open c_jug;
      fetch c_jug into i;
      if c_jug%found then
        close c_jug;
        -- Elimina el registre de la taula JUGADORS_EQUIPS que t l'id del jugador que s'ha passat per parmetre.
        delete from jugadors_equips where jugador = p_id;
        commit;
        -- Elimina el registre de la taula JUGADORS_REPRESENTANTS que t l'id del jugador que s'ha passat per parmetre.
        delete from jugadors_representants where jugador = p_id;
        -- Elimina el registre de la taula JUGADORS que t l'id que s'ha passat per parmetre.
        commit;
        delete from jugadors where id = p_id;
        commit;
      else 
        pkg_control.prc_exec_error(v_id,'ERROR: EL JUGADOR QUE HAS INTRODUIT ('||p_id||') NO EXISTEIX A LA BASE DE DADES.');
      end if;
		end if;

		-- Inserta a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		pkg_control.prc_exec_fi(v_id);
  exception
		-- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
    -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
  	when others then
	    pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));
  end prc_jugador_baixa;


	--------------------------
	-- PRC_JUGADOR_MODIFICA --
	--------------------------
	procedure prc_jugador_modifica (
		p_id                 varchar2,
    p_nacionalitat       varchar2,
    p_federacio          varchar2,
    p_num_federat        number,
    p_nom                varchar2,
    p_sexe               varchar2,  -- 1: Home i 2: Dona
    p_data_naixement     date,
    p_alada             number,
    p_pes                number,
    p_posicio            number,  -- 1: Base, 2: Escorta, 3: Aler, 4: Aler-Pivot i 5: Pivot
    p_representant       varchar2,
    p_url_personal       varchar2,
    p_url_promocio       varchar2,
    p_ind_activitat      varchar2,  -- A: Actiu i R: Retirat
    p_estat              varchar2,  -- A: Alta i B: Baixa
    p_sub_estat          varchar2,  -- BM: Baixa mdica i MP: Motius personals
    p_data_disp_estimada date
	) is

  -- Variable que s'utilitza per emmagatzemar l'identificador nic del log que retorna la funci FUN_REG_EXEC.
  v_id number;
  r_jug jugadors%rowtype;
  -- Cursor que retorna el representant de la taula JUGADORS_REPRESENTANTS del jugador que t l'id igual al passat per parmetre.
  cursor c_jr is
    select representant from jugadors_representants
     where jugador = p_id
       for update of representant;
  -- Cursor que retorna el registre de la taula JUGADORS del jugador que t l'id igual al passat per parmetre.
  cursor c_jug is
    select * from jugadors
     where id = p_id
       for update of nom;
  i c_jug%rowtype;

  begin
	  -- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_JUGADOR_MODIFICA','PAR1: '||p_id||
                                                                 ' - PAR2: '||p_nacionalitat||
                                                                 ' - PAR3: '||p_federacio||
                                                                 ' - PAR4: '||p_num_federat||
                                                                 ' - PAR5: '||p_nom||
                                                                 ' - PAR6: '||p_sexe||
                                                                 ' - PAR7: '||p_data_naixement||
                                                                 ' - PAR8: '||p_alada||
                                                                 ' - PAR9: '||p_pes||
                                                                 ' - PAR10: '||p_posicio||
                                                                 ' - PAR11: '||p_url_personal||
                                                                 ' - PAR12: '||p_url_promocio||
                                                                 ' - PAR13: '||p_ind_activitat||
                                                                 ' - PAR14: '||p_estat||
                                                                 ' - PAR15: '||p_sub_estat||
                                                                 ' - PAR16: '||p_data_disp_estimada);
    if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment per no s'actualitza cap registre a les taules JUGADORS_REPRESENTANTS i JUGADORS.
		  pkg_control.prc_exec_test(v_id);
	  else
      open c_jug;
        fetch c_jug into i;
        if c_jug%found then
          close c_jug;
          -- Si el representant ha canviat, actualitza a la taula JUGADORS_REPRESENTANTS el nou codi de representant.
          for i in c_jr loop
            if i.representant != nvl(p_representant,i.representant) then
              update jugadors_representants set representant = p_representant where current of c_jr;
            end if;
          end loop;
          for i in c_jug loop
            -- Els valors que s'han deixat en blanc al cridar el procediment es deixen com estan a la taula JUGADORS.
            r_jug.id                  :=   p_id;
            r_jug.nacionalitat        :=   nvl(p_nacionalitat,i.nacionalitat);
            r_jug.federacio           :=   nvl(p_federacio,i.federacio);
            r_jug.num_federat         :=   nvl(p_num_federat,i.num_federat);
            r_jug.nom                 :=   nvl(p_nom,i.nom);
            r_jug.sexe                :=   nvl(p_sexe,i.sexe);
            r_jug.data_naixement      :=   nvl(p_data_naixement,i.data_naixement);
            r_jug.alada              :=   nvl(p_alada,i.alada);
            r_jug.pes                 :=   nvl(p_pes,i.pes);
            r_jug.posicio             :=   nvl(p_posicio,i.posicio);
            r_jug.url_personal        :=   nvl(p_url_personal,i.url_personal);
            r_jug.url_promocio        :=   nvl(p_url_promocio,i.url_promocio);
            r_jug.ind_activitat       :=   nvl(p_ind_activitat,i.ind_activitat);
            r_jug.estat               :=   nvl(p_estat,i.estat);
            r_jug.sub_estat           :=   nvl(p_sub_estat,i.sub_estat);
            r_jug.data_disp_estimada  :=   nvl(p_data_disp_estimada,i.data_disp_estimada);
        
            update jugadors set row = r_jug where current of c_jug;
          end loop;
      else
        pkg_control.prc_exec_error(v_id,'ERROR: EL JUGADOR QUE HAS INTRODUIT ('||p_id||') NO EXISTEIX A LA BASE DE DADES.');
      end if;
    end if;
    -- Confirma les actualitzacions a les taules JUGADORS_REPRESENTANTS i JUGADORS.
    commit;
		-- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		pkg_control.prc_exec_fi(v_id);
  exception
		-- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
    -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
  	when others then
      pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));
  end prc_jugador_modifica;

end pkg_jugador;
/

spool off;
