/*
	NOM PAQUET: PKG_CONTRACTE
	
	FUNCIONALITAT: Permet realitzar altes, baixes i modificacions de contractes al sistema.

*/

spool 4_3_Creacio_Pkg_Contracte.log;
  
----------------
-- CAPAALERA --
----------------
create or replace
package pkg_contracte as

  -------------------
  -- PKG_CONTRACTE --
  -------------------

  -- Variable que cont el nom del paquet que s'utilitza per a registar-lo a la taula LOG_PROC_EXEC.
	pck_nom varchar2(13):='PKG_CONTRACTE';

	-- Variable que s'utilitza per determinar si el sistema es troba en mode TEST.
	v_test_actiu boolean;

	-- Procediment que s'utilitza per donar d'alta un nou contracte al sistema.
	procedure prc_contracte_alta (
    p_data_inici         date,
    p_jugador            varchar2,
    p_equip_comprador    varchar2,
    p_equip_venedor      varchar2,
    p_durada             number,
    p_salari_brut        number,
    p_compensacio_equip  number,
    p_moneda             varchar2
	);

	-- Procediment que s'utilitza per donar de baixa al sistema el contracte que es correspon als parmetres passats.
	procedure prc_contracte_baixa (
    p_data_inici date,
    p_jugador varchar2,
    p_equip_comprador varchar2,
    p_equip_venedor varchar2
	);

	-- Procediment que s'utilitza per modificar les dades del contracte que es correspon als parmetres passats.
	procedure prc_contracte_modifica (
		p_data_inici date,
    p_data_fi date,
    p_jugador varchar2,
    p_equip_comprador varchar2,
    p_equip_venedor varchar2,
    p_durada number,
    p_salari_brut number,
    p_compensacio_equip number,
    p_moneda varchar2
	);

  -- Procediment que s'utilitza per cancel lar un contracte modificant noms la seva data de finalitzaci.
  procedure prc_contracte_finalitza (
    p_data_inici date,
    p_data_fi date default trunc(sysdate),
    p_jugador varchar2,
    p_equip_comprador varchar2,
    p_equip_venedor varchar2
	);

end pkg_contracte;
/

--------------------
-- COS DEL PAQUET --
--------------------
create or replace
package body pkg_contracte as

  ------------------------
  -- PRC_CONTRACTE_ALTA --
  ------------------------
  procedure prc_contracte_alta (
    p_data_inici         date,
    p_jugador            varchar2,
    p_equip_comprador    varchar2,
    p_equip_venedor      varchar2,
    p_durada             number,
    p_salari_brut        number,
    p_compensacio_equip  number,
    p_moneda             varchar2
	) is

  -- Variable que s'utilitza per emmagatzemar l'identificador nic del log que retorna la funci FUN_REG_EXEC.
  v_id number;
  r_con contractes%rowtype;
  -- Cursor que retorna la data d'inici, equip comprador i la ltima data de finalitzaci del contracte que t data fi
  -- inferior a la data passada com a parmetre al cursor.
  cursor c_con_act (p_data date) is
    select distinct c.data_inici, e.nom equip_comprador,
           first_value(c.data_fi) over (partition by c.jugador order by c.data_fi desc) data_fi
      from contractes c, equips e
     where c.equip_comprador = e.id
       and c.jugador = p_jugador
       and c.data_fi < p_data;
  r_con_act c_con_act%rowtype;
  
  -- Cursor que retorna l'equip del jugador passat per parmetre
  cursor c_jug_equips is
    select equip 
     from jugadors_equips
     where jugador = p_jugador;
  r_jug_equips c_jug_equips%rowtype;

  begin
    -- Regitra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_CONTRACTE_ALTA','PAR1: '||p_data_inici||
                                                               ' - PAR2: '||p_jugador||
                                                               ' - PAR3: '||p_equip_comprador||
                                                               ' - PAR4: '||p_equip_venedor||
                                                               ' - PAR5: '||p_durada||
                                                               ' - PAR6: '||p_salari_brut||
                                                               ' - PAR7: '||p_compensacio_equip||
                                                               ' - PAR8: '||p_moneda);
	  if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment per no s'insereix cap registre a la taula CONTRACTES.
	    pkg_control.prc_exec_test(v_id);
	  else
      r_con.data_inici          :=   nvl(trunc(p_data_inici),trunc(sysdate));  -- Si no s'introdueix una data d'inici s'insereix la data actual.
      r_con.jugador             :=   p_jugador;
      r_con.equip_comprador     :=   p_equip_comprador;
      r_con.equip_venedor       :=   p_equip_venedor;
      r_con.durada              :=   nvl(p_durada,365);  -- Si el valor s nul, la taula insereix 365 dies per defecte .
      r_con.salari_brut         :=   p_salari_brut;
      r_con.compensacio_equip   :=   p_compensacio_equip;
      r_con.total_operacio      :=   nvl(r_con.salari_brut,0) + nvl(r_con.compensacio_equip,0);  -- Calcula el total de l'operaci.
      r_con.moneda              :=   nvl(p_moneda,'EUR');  -- Si el valor s nul, la taula insereix EUR per defecte.
      r_con.data_fi             :=   trunc(r_con.data_inici) + nvl(r_con.durada,365);  -- Data d'inici ms la durada del contracte.
      open c_con_act(r_con.data_fi);
      fetch c_con_act into r_con_act;
        if c_con_act%found then
          -- Si el jugador t un contracte vigent es genera un missatge d'error informant de la situaci.
          pkg_control.prc_exec_error(v_id,'ERROR: EL JUGADOR '||p_jugador||' TE UN CONTRACTE VIGENT DES DEL '||
                                          to_char(r_con.data_inici,'dd/mm/yyyy')||' AMB EL '||r_con_act.equip_comprador||
                                          ' QUE NO FINALITZA FINS EL DIA '||to_char(r_con_act.data_fi,'dd/mm/yyyy')||'.');
        else
        -- Insereix a la taula CONTRACTES el registre amb les dades introduides del nou contracte.
        insert into contractes values r_con;
        -- Obrim cursor per assegurar-nos que el jugador no estigui vinculat a cap equip.
        -- Si ho est, s'actualitza pel nou equip, en cas contrari es dona d'alta un registre amb el id del jugador i el de l'equip
        -- a la taula JUGADORS_EQUIPS.
        open c_jug_equips;
          fetch c_jug_equips into r_jug_equips;
            if c_jug_equips%found then 
              update jugadors_equips set equip = p_equip_comprador where jugador = p_jugador;
            else 
              insert into jugadors_equips values (p_jugador,p_equip_comprador);
            end if;
        close c_jug_equips;
        end if;  
      close c_con_act;
    end if;
    -- Confirma la inserci del nou registre a la taula CONTRACTES i l'actualitzaci o inserci a la taula JUGADORS_EQUIPS.
    commit;
		-- Inserta a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		pkg_control.prc_exec_fi(v_id);
  exception
	  -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
    -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
    when others then
	    pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));
  end prc_contracte_alta;


  -------------------------
  -- PRC_CONTRACTE_BAIXA --
  -------------------------
  procedure prc_contracte_baixa (
    p_data_inici date,
    p_jugador varchar2,
    p_equip_comprador varchar2,
    p_equip_venedor varchar2
	) is

  -- Variable que s'utilitza per emmagatzemar l'identificador nic del log que retorna la funci FUN_REG_EXEC.
  v_id number;
  -- Cursor que retorna el registre del contracte que es correspon amb els parmetres passats.
  cursor c_con is
    select * from contractes
     where data_inici = p_data_inici
       and jugador = p_jugador
       and equip_comprador = p_equip_comprador
       and equip_venedor = p_equip_venedor
       for update of data_inici;
  i c_con%rowtype;

  -- Cursor que retorna l'equip del jugador passat per parmetre
  cursor c_jug_equips is
    select equip 
     from jugadors_equips
     where jugador = p_jugador
       for update of jugador;

  begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
		v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_CONTRACTE_BAIXA','PAR1: '||p_data_inici||
                                                                ' - PAR2: '||p_jugador||
                                                                ' - PAR3: '||p_equip_comprador||
                                                                ' - PAR4: '||p_equip_venedor);
    if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment per no s'elimina cap registre de taula CONTRACTES.
			pkg_control.prc_exec_test(v_id);
		else
      open c_con;
      fetch c_con into i;
        if c_con%found then
          close c_con;
          -- S'esborra el registre de la taula JUGADORS_EQUIPS que cont el id del jugador passat per parmetre.
          for i in c_jug_equips loop
            delete from jugadors_equips where current of c_jug_equips;
          end loop;
          -- Es confirma l'eliminaci del registre de la taula JUGADORS_EQUIPS.
          commit;
          -- Elimina el registre de la taula CONTRACTES que es correspon amb els parmetres passats..
          for i in c_con loop
            delete from contractes where current of c_con;
          end loop;
          commit;
        else
          pkg_control.prc_exec_error(v_id,'ERROR: EL CONTRACTE QUE HAS INTRODUIT NO EXISTEIX A LA BASE DE DADES.');
        end if;
		end if;
		-- Inserta a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		pkg_control.prc_exec_fi(v_id);
  exception
		-- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
    -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
  	when others then
	    pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));
  end prc_contracte_baixa;


  ----------------------------
  -- PRC_CONTRACTE_MODIFICA --
  ----------------------------
  procedure prc_contracte_modifica (
		p_data_inici date,
    p_data_fi date,
    p_jugador varchar2,
    p_equip_comprador varchar2,
    p_equip_venedor varchar2,
    p_durada number,
    p_salari_brut number,
    p_compensacio_equip number,
    p_moneda varchar2
	) is

  -- Variable que s'utilitza per emmagatzemar l'identificador nic del log que retorna la funci FUN_REG_EXEC.
  v_id number;
  r_con contractes%rowtype;
  -- Cursor que retorna el registre de la taula CONTRACTES que coincideix amb els parmetres passats.
  cursor c_con is
    select * from contractes
     where data_inici = p_data_inici
       and jugador = p_jugador
       and equip_venedor = p_equip_venedor
       and equip_comprador = p_equip_comprador
       for update of data_fi;
  i c_con%rowtype;
       
  -- Cursor que retorna l'equip del jugador passat per parmetre
  cursor c_jug_equips is
    select equip 
     from jugadors_equips
     where jugador = p_jugador;
  r_jug_equips c_jug_equips%rowtype;

  begin
    -- Regitra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
    v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_CONTRACTE_MODIFICA','PAR1: '||p_data_inici||
                                                                   ' - PAR2: '||p_data_fi||
                                                                   ' - PAR3: '||p_jugador||
                                                                   ' - PAR4: '||p_equip_comprador||
                                                                   ' - PAR5: '||p_equip_venedor||
                                                                   ' - PAR6: '||p_durada||
                                                                   ' - PAR7: '||p_salari_brut||
                                                                   ' - PAR8: '||p_compensacio_equip||
                                                                   ' - PAR9: '||p_moneda);
	  if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment per no modifica cap registre a la taula CONTRACTES.
	    pkg_control.prc_exec_test(v_id);
  	else
      open c_con;
      fetch c_con into i;
        if c_con%found then
          close c_con;
          -- Insereix a la taula CONTRACTES el registre amb les dades introduides del nou contracte.
          for i in c_con loop
            r_con.data_inici          :=   nvl(p_data_inici,i.data_inici);
            r_con.jugador             :=   nvl(p_jugador,i.jugador);
            r_con.equip_comprador     :=   nvl(p_equip_comprador,i.equip_comprador);
            r_con.equip_venedor       :=   nvl(p_equip_venedor,i.equip_venedor);
            r_con.durada              :=   nvl(p_durada,i.durada);
            r_con.salari_brut         :=   nvl(p_salari_brut,i.salari_brut);
            r_con.compensacio_equip   :=   nvl(p_compensacio_equip,i.compensacio_equip);
            r_con.total_operacio      :=   nvl(p_salari_brut,i.salari_brut) + nvl(p_compensacio_equip,i.compensacio_equip);
            r_con.moneda              :=   nvl(p_moneda,i.moneda);
            r_con.data_fi             :=   nvl(p_data_fi,(i.data_inici + nvl(p_durada,i.durada)));
            -- Actualitza la data fi del contracte passat per parmetre.
            update contractes set row = r_con where current of c_con;
          end loop;
          -- Es comprova si l'equip ha canviat, si ho ha fet s'actualitza la taula JUGADORS_EQUIP amb el nou equip.
          open c_jug_equips;
          fetch c_jug_equips into r_jug_equips;
            if r_jug_equips.equip != r_con.equip_comprador then
              update jugadors_equips set equip = r_con.equip_comprador;
            end if;
          close c_jug_equips;
        else 
          pkg_control.prc_exec_error(v_id,'ERROR: EL CONTRACTE QUE HAS INTRODUIT NO EXISTEIX A LA BASE DE DADES.');
        end if; 
    end if;
    -- Confirma l'actualitzaci del registre a la taula CONTRACTES.
    commit;
	  -- Inserta a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
  	pkg_control.prc_exec_fi(v_id);
  exception
	  -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
    -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
  	when others then
	    pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));
  end prc_contracte_modifica;


  -----------------------------
  -- PRC_CONTRACTE_FINALITZA --
  -----------------------------
  procedure prc_contracte_finalitza (
    p_data_inici date,
    p_data_fi date default trunc(sysdate),
    p_jugador varchar2,
    p_equip_comprador varchar2,
    p_equip_venedor varchar2
	) is

  -- Variable que s'utilitza per emmagatzemar l'identificador nic del log que retorna la funci FUN_REG_EXEC.
  v_id number;
  r_con contractes%rowtype;
  -- Cursor que retorna la data de fi del registre de la taula CONTRACTES que coincideix amb els parmetres passats.
  cursor c_con is
    select *
      from contractes
     where data_inici = p_data_inici
       and jugador = p_jugador
       and equip_venedor = p_equip_venedor
       and equip_comprador = p_equip_comprador
       for update of data_fi;
  i c_con%rowtype;

  begin
    -- Regitra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_CONTRACTE_FINALITZA','PAR1: '||p_data_inici||
                                                                    ' - PAR2: '||p_data_fi||
                                                                    ' - PAR3: '||p_jugador||
                                                                    ' - PAR4: '||p_equip_comprador||
                                                                    ' - PAR5: '||p_equip_venedor);
	  if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment per no dna de baixa cap contracte a la taula CONTRACTES.
		  pkg_control.prc_exec_test(v_id);
	  else
      open c_con;
      fetch c_con into i;
        if c_con%found then
          close c_con;
          for i in c_con loop
            -- Actualitza la data fi del contracte passat per parmetre.
            update contractes set data_fi = p_data_fi, durada = (p_data_fi-data_inici) where current of c_con;
          end loop;
        else
          pkg_control.prc_exec_error(v_id,'ERROR: EL CONTRACTE QUE HAS INTRODUIT NO EXISTEIX A LA BASE DE DADES.'); 
        end if;
    end if;
    -- Confirma l'actualitzaci del registre corresponent al contracte tractat.
    commit;
		-- Inserta a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		pkg_control.prc_exec_fi(v_id);
	exception
	  -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
    -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
  	when others then
	    pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));
  end prc_contracte_finalitza;

end pkg_contracte;
/

spool off;
