/*
	NOM PAQUET: PKG_CONTRACTE
	
	FUNCIONALITAT: Permet realitzar altes, baixes i modificacions de partits al sistema.

*/

spool 4_4_Creacio_Pkg_Partit.log;
  
----------------
-- CAPAALERA --
----------------
create or replace
package pkg_partit as

  ----------------
  -- PKG_PARTIT --
  ----------------

  -- Variable que cont el nom del paquet que s'utilitza per a registar-lo a la taula LOG_PROC_EXEC.
	pck_nom varchar2(10):='PKG_PARTIT';

	-- Variable que s'utilitza per determinar si el sistema es troba en mode TEST.
	v_test_actiu boolean;

	-- Procediment que s'utilitza per donar d'alta un nou partit al sistema.
	procedure prc_partit_alta (
    p_data             date,
    p_lloc             varchar2,
    p_equip_local      varchar2,
    p_equip_visitant   varchar2,
    p_competicio       number
	);

	-- Procediment que s'utilitza per donar de baixa el partit que amb l'id passat per parmetre.
	procedure prc_partit_baixa (
    p_id number
	);

	-- Procediment que s'utilitza per modificar les dades del partit que es correspon als parmetres passats.
	procedure prc_partit_modifica (
		p_id number,
    p_data date,
    p_lloc varchar2,
    p_equip_local varchar2,
    p_equip_visitant varchar2,
    p_competicio number
	);

end pkg_partit;
/

--------------------
-- COS DEL PAQUET --
--------------------
create or replace
PACKAGE BODY PKG_PARTIT AS

  ---------------------
  -- PRC_PARTIT_ALTA --
  ---------------------
  procedure prc_partit_alta (
    p_data             date,
    p_lloc             varchar2,
    p_equip_local      varchar2,
    p_equip_visitant   varchar2,
    p_competicio       number
	) is

  -- Variable que s'utilitza per emmagatzemar l'identificador nic del log que retorna la funci FUN_REG_EXEC.
  v_id number;
  r_par partits%rowtype;

  begin
    -- Regitra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_PARTIT_ALTA','PAR1: '||p_data||
                                                            ' - PAR2: '||p_lloc||
                                                            ' - PAR3: '||p_equip_local||
                                                            ' - PAR4: '||p_equip_visitant||
                                                            ' - PAR5: '||p_competicio);
	  if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment per no s'insereix cap registre a la taula PARTITS.
	    pkg_control.prc_exec_test(v_id);
	  else
      r_par.id               :=   seq_partits.nextval;
      r_par.data             :=   p_data;
      r_par.lloc             :=   p_lloc;
      r_par.equip_local      :=   p_equip_local;
      r_par.equip_visitant   :=   p_equip_visitant;
      r_par.competicio       :=   p_competicio;
      -- Insereix a la taula PARTITS el registre amb les dades introduides del nou partit.
      insert into partits values r_par;
    end if;
    -- Confirma la inserci del nou registre a la taula PARTITS.
    commit;
		-- Inserta a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		pkg_control.prc_exec_fi(v_id);
  exception
	  -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
    -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
    when others then
	    pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));
  end prc_partit_alta;


  ----------------------
  -- PRC_PARTIT_BAIXA --
  ----------------------
  procedure prc_partit_baixa (
    p_id number
	) is

  -- Variable que s'utilitza per emmagatzemar l'identificador nic del log que retorna la funci FUN_REG_EXEC.
  v_id number;
  
  cursor c_par is
    select * from partits
     where id = p_id;
  i c_par%rowtype;

  begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
		v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_PARTIT_BAIXA','PAR1: '||p_id);
    if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment per no s'elimina cap registre de les taules: JUGADORS_PARTITS, ARBITRES_PARTITS i PARTITS.
			pkg_control.prc_exec_test(v_id);
		else
      open c_par;
      fetch c_par into i;
        if c_par%found then
          close c_par;
          -- Elimina el registre de la taula JUGADORS_PARTITS que t l'id del partit que s'ha passat per parmetre.
          delete from jugadors_partits where partit = p_id;
          commit;
          -- Elimina el registre de la taula ARBITRES_PARTITS que t l'id del partit que s'ha passat per parmetre.
          delete from arbitres_partits where partit = p_id;
          -- Elimina el registre de la taula PARTITS que t l'id que s'ha passat per parmetre.
          commit;
          delete from partits where id = p_id;
          commit;
        else 
          pkg_control.prc_exec_error(v_id,'ERROR: EL PARTIT QUE HAS INTRODUIT ('||p_id||') NO EXISTEIX A LA BASE DE DADES.');
        end if;
      end if;
  end prc_partit_baixa;


  -------------------------
  -- PRC_PARTIT_MODIFICA --
  -------------------------
  procedure prc_partit_modifica (
		p_id number,
    p_data date,
    p_lloc varchar2,
    p_equip_local varchar2,
    p_equip_visitant varchar2,
    p_competicio number
	) is

  -- Variable que s'utilitza per emmagatzemar l'identificador nic del log que retorna la funci FUN_REG_EXEC.
  v_id number;
  r_par partits%rowtype;
  -- Cursor que retorna el registre de la taula PARTITS del partit que t l'id igual al passat per parmetre.
  cursor c_par is
    select * from partits
     where id = p_id
       for update of id;
  i partits%rowtype;

  begin
	  -- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_PARTIT_MODIFICA','PAR1: '||p_id||
                                                                ' - PAR2: '||p_data||
                                                                ' - PAR3: '||p_lloc||
                                                                ' - PAR4: '||p_equip_local||
                                                                ' - PAR5: '||p_equip_visitant||
                                                                ' - PAR6: '||p_competicio);
    if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment per no s'actualitza cap registre a la taula PARTITS.
		  pkg_control.prc_exec_test(v_id);
	  else
      open c_par;
      fetch c_par into i;
        if c_par%found then
          -- Els valors que s'han deixat en blanc al cridar el procediment es deixen com estan a la taula PARTITS.
          r_par.id               :=   nvl(p_id,i.id);
          r_par.data             :=   nvl(p_data,i.data);
          r_par.lloc             :=   nvl(p_lloc,i.lloc);
          r_par.equip_local      :=   nvl(p_equip_local,i.equip_local);
          r_par.equip_visitant   :=   nvl(p_equip_visitant,i.equip_visitant);
          r_par.competicio       :=   nvl(p_competicio,i.competicio);
        
          update partits set row = r_par where current of c_par;
        else 
          pkg_control.prc_exec_error(v_id,'ERROR: EL PARTIT QUE HAS INTRODUIT ('||p_id||') NO EXISTEIX A LA BASE DE DADES.');
        end if;
      close c_par;
    end if;
    -- Confirma les actualitzacions a la taula PARTITS.
    commit;
		-- Inserta a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		pkg_control.prc_exec_fi(v_id);
  exception
		-- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
    -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
  	when others then
      pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));
  end prc_partit_modifica;

end pkg_partit;
/

spool off;
