/*
	NOM PAQUET: PKG_COSULTES
	
	FUNCIONALITAT: Permet realitzar consultes al sistema que retornen els resultats en forma de informes.

*/

spool 4_5_Creacio_Pkg_Consultes.log;
  
----------------
-- CAPAALERA --
----------------
create or replace
package pkg_consultes as

  -------------------
  -- PKG_CONSULTES --
  -------------------

	-- Variable que cont el nom del paquet que s'utilitza per a registar-lo a la taula LOG_PROC_EXEC.
	pck_nom varchar2(13):='PKG_CONTRACTE';

	-- Variable que s'utilitza per determinar si el sistema es troba en mode TEST.
	v_test_actiu boolean;

  -- Procediment que retorna el llistat de tots els jugadors d'una competici donada amb totes les seves dades,
  -- incloent la data de finalitzaci del contracte actual.
  procedure prc_jugadors_competicio (
    p_competicio_id number
  );

  -- Procediment que retorna el llistat de tots els equips d'una competici ordenats pel nombre de punts totals
  -- a favor en la temporada actual.
  procedure prc_equips_competicio (
    p_competicio_id number
  );

  -- Procediment que donat un mbit, un gnere i una modalitat de competici, retorna el llistat dels 5 millors
  -- jugadors per posici en funci de la seva valoraci.
  procedure prc_top5_jugadors (
    p_ambit_id varchar2,
    p_genere_id varchar2,
    p_modalitat_id varchar2
  );

  -- Procediment que donat un any i un representant de jugadors, retorna el nmero de contractes signats i el valor econmic
  -- total de cadascun d'ells.
  procedure prc_contractes_representant (
    p_any varchar2,
    p_representant_id varchar2
  );

  -- Procediment que donat un any concret, retorna el llistat dels 10 equips que ms diners han gastat en adquisici de jugadors,
  -- ordenats de ms a menys.
  procedure prc_top10_equips_gastat (
    p_any varchar2
  );

  -- Procediment que donat un pas, un gnere i una modalitat, retorna el llistat de jugadors que acaben contracte al final de la
  -- present temporada o que estan en actiu per sense equip.
  procedure prc_jugadors_lliures (
    p_pais_id varchar2,
    p_genere_id varchar2,
    p_modalitat_id varchar2);

end pkg_consultes;
/

--------------------
-- COS DEL PAQUET --
--------------------
create or replace
package body pkg_consultes is

  -----------------------------
  -- PRC_JUGADORS_COMPETICIO --
  -----------------------------
  procedure prc_jugadors_competicio (
    p_competicio_id number
  ) is

  -- Variable que s'utilitza per emmagatzemar l'identificador nic del log que retorna la funci FUN_REG_EXEC.
  v_id number;
  v_nom_com varchar2(50);

  --Cursor que retorna el nom de la competici que es correspon amb el id introduit com a parmetre.
  cursor c_nom_com is
    select distinct nom
      from competicions
     where id = p_competicio_id;

  -- Cursor que retorna totes les dades dels jugadors, incloent la data finalitzaci del contracte, de la competici passada com a parmetre.
  cursor c_jug_com is
    select distinct j.id id_jugador, pc.descripcio nacionalitat, n.descripcio federacio, j.num_federat, j.nom, j.sexe, j.data_naixement, j.alada,
           j.pes, p.descripcio posicio, r.nom representant, e.nom equip, j.url_personal, j.url_promocio, j.ind_activitat, j.estat, j.sub_estat,
           last_value(c.data_fi) over (partition by c.jugador order by c.data_fi asc) data_fi_contracte
      from jugadors j, posicions p, equips e, representants_jugadors r, contractes c, partits pa, nacionalitats n,
           jugadors_representants jr,  jugadors_equips je, jugadors_partits jp, paisos_continents pc
     where pa.competicio = p_competicio_id
       and jp.jugador = j.id
       and jp.partit = pa.id
       and jr.jugador = j.id
       and jr.representant = r.id
       and je.jugador = j.id
       and je.equip = e.id
       and j.posicio = p.id
       and c.jugador = j.id
       and j.nacionalitat = pc.id
       and j.nacionalitat = n.id
     order by j.nom;

  r_jug_com c_jug_com%rowtype;

  begin
    -- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_JUGADORS_COMPETICIO','PAR1: '||p_competicio_id);
	  if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment, ja que el procediment s de consulta.
	    pkg_control.prc_exec_test(v_id);
    end if;

    -- Augmentem el tamany del buffer de l'output suficient perqu cpiga tot el missatge.
    dbms_output.enable(1000000);
    -- Assigna a la variable v_nom_com el nom de la competici amb el id passat com a parmetre.
    open c_nom_com;
      fetch c_nom_com into v_nom_com;
    -- Es tanca el cursor.
    close c_nom_com;

    dbms_output.put_line('++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.put_line('LLISTAT DE JUGADORS DE LA COMPETICI: '||v_nom_com);
    dbms_output.put_line('++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.new_line;

		-- S'obre el cursor amb les dades dels jugadors de la competici
		open c_jug_com;
		fetch c_jug_com into r_jug_com;
      if c_jug_com%found then
        close c_jug_com;
        for i in c_jug_com loop
          -- Si el cursor retorna registres s'escriuen al buffer de l'output.
          dbms_output.put_line('ID JUGADOR: '||i.id_jugador);
		    	dbms_output.put_line('NOM: '||i.nom);
          dbms_output.put_line('SEXE: '||i.sexe);
          dbms_output.put_line('NACIONALITAT: '||i.nacionalitat);
          dbms_output.put_line('FEDERACI: '||i.federacio);
          dbms_output.put_line('NMERO DE FEDERAT: '||i.num_federat);
          dbms_output.put_line('DATA DE NAIXAMENT: '||to_char(i.data_naixement,'dd/mm/yyyy'));
          dbms_output.put_line('ALADA: '||i.alada);
          dbms_output.put_line('PES: '||i.pes);
          dbms_output.put_line('POSICIO: '||i.posicio);
          dbms_output.put_line('REPRESENTANT: '||i.representant);
    			dbms_output.put_line('EQUIP: '||i.equip);
    			dbms_output.put_line('URL PERSONAL: '||i.url_personal);
          dbms_output.put_line('URL PROMOCI: '||i.url_promocio);
          dbms_output.put_line('INDICADOR D''ACTIVITAT: '||i.ind_activitat);
          dbms_output.put_line('ESTAT: '||i.estat);
          dbms_output.put_line('SUBESTAT: '||i.sub_estat);
          dbms_output.put_line('DATA FI DE CONTRACTE: '||to_char(i.data_fi_contracte,'dd/mm/yyyy'));
          dbms_output.new_line;
        end loop;
      else
        dbms_output.put_line('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
        dbms_output.put_line('LA CONSULTA NO HA RETORNAT CAP JUGADOR PER AQUESTA COMPETICIO.');
        dbms_output.put_line('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
  		  dbms_output.new_line;
		  end if;
		-- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		pkg_control.prc_exec_fi(v_id);
  exception
	  -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
    -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
    when others then
	    pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));
  end prc_jugadors_competicio;


  ---------------------------
  -- PRC_EQUIPS_COMPETICIO --
  ---------------------------
  procedure prc_equips_competicio (
    p_competicio_id number
  ) is

  -- Variable que s'utilitza per emmagatzemar l'identificador nic del log que retorna la funci FUN_REG_EXEC.
  v_id number;

  --Cursor que retorna les dades de la competici que es correspon amb el id introduit com a parmetre.
  cursor c_comp is
    select distinct c.nom, c.temporada, a.descripcio ambit, g.descripcio genere, m.descripcio modalitat,
           p.descripcio pais_continent, ca.descripcio categoria
      from competicions c, ambits a, generes g, modalitats m, paisos_continents p, categories ca
     where c.id = p_competicio_id
       and c.ambit = a.id
       and c.genere = g.id
       and c.modalitat = m.id
       and c.pais_continent = p.id
       and c.categoria = ca.id;

  r_comp c_comp%rowtype;

  -- Cursor que retorna totes les dades dels equips ordenats pel nombre de punts a favor de la competici passada com a parmetre.
  cursor c_equip_com is
    select e.nom equip, sum(es.pt) punts
      from equips e, partits p, jugadors_equips je, jugadors_partits jp, estadistiques es
     where p.competicio = p_competicio_id
       and es.partit = p.id
       and es.jugador = je.jugador
       and jp.jugador = es.jugador
       and jp.partit = es.partit
       and e.id = je.equip
     group by e.nom
     order by sum(es.pt);

  r_equip_com c_equip_com%rowtype;

  begin
    -- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_EQUIPS_COMPETICIO','PAR1: '||p_competicio_id);
	  if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment, ja que el procediment s de consulta.
	    pkg_control.prc_exec_test(v_id);
    end if;

    -- Augmentem el tamany del buffer de l'output suficient perqu cpiga tot el missatge.
    dbms_output.enable(1000000);
    -- Obrim el cursor amb les dades de la competici.
    open c_comp;
      fetch c_comp into r_comp;
    -- Es tanca el cursor.
    close c_comp;
    dbms_output.put_line('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.put_line('RANKIG DE PUNTS DELS EQUIPS DE LA COMPETICI: '||r_comp.nom);
    dbms_output.put_line('TEMPORADA: '||r_comp.temporada);
    dbms_output.put_line('AMBIT: '||r_comp.ambit);
    dbms_output.put_line('GENERE: '||r_comp.genere);
    dbms_output.put_line('MODALITAT: '||r_comp.modalitat);
    dbms_output.put_line('PAIS O CONTINENT: '||r_comp.pais_continent);
    dbms_output.put_line('CATEGORIA: '||r_comp.categoria);
    dbms_output.put_line('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.new_line;
		-- S'obre el cursor amb les dades dels equips dels equips de la competici.
		open c_equip_com;
		fetch c_equip_com into r_equip_com;
    -- Si el cursor retorna registres s'escriuen al buffer de l'output.
    if c_equip_com%found then
      close c_equip_com;
      for i in c_equip_com loop
        dbms_output.put_line('EQUIP: '||i.equip);
        dbms_output.put_line('PUNTS: '||i.punts);
        dbms_output.new_line;
      end loop;
  else
      dbms_output.put_line('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
      dbms_output.put_line('LA CONSULTA NO HA RETORNAT CAP EQUIP PER AQUESTA COMPETICIO.');
      dbms_output.put_line('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
  		dbms_output.new_line;
		end if;
		-- Es tanca el cursor.
		close c_equip_com;
		-- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		pkg_control.prc_exec_fi(v_id);
  exception
	  -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
    -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
    when others then
	    pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));
  end prc_equips_competicio;


  -----------------------
  -- PRC_TOP5_JUGADORS --
  -----------------------
  procedure prc_top5_jugadors (
    p_ambit_id varchar2,
    p_genere_id varchar2,
    p_modalitat_id varchar2
  ) is

  -- Variable que s'utilitza per emmagatzemar l'identificador nic del log que retorna la funci FUN_REG_EXEC.
  v_id number;
  v_ambit varchar2(20);
  v_genere varchar2(20);
  v_modalitat varchar2(40);

  -- Cursor que retorna el nom de l'mbit que es correspon amb l'id introduit per parmetre.
  cursor c_ambit is
    select descripcio ambit from ambits where id = p_ambit_id;

  -- Cursor que retorna el nom del gnere que es correspon amb l'id introduit per parmetre.
  cursor c_genere is
    select descripcio genere from generes where id = p_genere_id;

  -- Cursor que retorna el nom de la modalitat que es correspon amb l'id introduit per parmetre.
  cursor c_modalitat is
    select descripcio modalitat from modalitats where id = p_modalitat_id;

  -- Cursor que retorna els 5 millors bases donats un mbit, un gnere i una modalitat.
  cursor c_top5_base is
    select distinct nom, vp
      from (select distinct j.nom, e.vp
              from partits p, jugadors j, estadistiques e
             where j.posicio = 1 -- codi de base
               and e.partit = p.id
               and e.jugador = j.id
               and p.competicio in (select distinct c.id
                                      from competicions c, ambits a, generes g, modalitats m
                                     where c.ambit = p_ambit_id
                                       and c.genere = p_genere_id
                                       and c.modalitat = p_modalitat_id
                                       and c.ambit = a.id
                                       and c.genere = g.id
                                       and c.modalitat = m.id)
              order by e.vp)
     where rownum < 6
     order by vp desc;

  r_top5_base c_top5_base%rowtype;

  -- Cursor que retorna els 5 millors escortes donats un mbit, un gnere i una modalitat.
  cursor c_top5_escorta is
    select distinct nom, vp
      from (select distinct j.nom, e.vp
              from partits p, jugadors j, estadistiques e
             where j.posicio = 2  -- codi d'escorta
               and e.partit = p.id
               and e.jugador = j.id
               and p.competicio in (select distinct c.id
                                      from competicions c, ambits a, generes g, modalitats m
                                     where c.ambit = p_ambit_id
                                       and c.genere = p_genere_id
                                       and c.modalitat = p_modalitat_id
                                       and c.ambit = a.id
                                       and c.genere = g.id
                                       and c.modalitat = m.id)
              order by e.vp)
     where rownum < 6
     order by vp desc;

  r_top5_escorta c_top5_escorta%rowtype;

  -- Cursor que retorna els 5 millors alers donats un mbit, un gnere i una modalitat.
  cursor c_top5_aler is
    select distinct nom, vp
      from (select distinct j.nom, e.vp
              from partits p, jugadors j, estadistiques e
             where j.posicio = 3 -- codi d'aler
               and e.partit = p.id
               and e.jugador = j.id
               and p.competicio in (select distinct c.id
                                      from competicions c, ambits a, generes g, modalitats m
                                     where c.ambit = p_ambit_id
                                       and c.genere = p_genere_id
                                       and c.modalitat = p_modalitat_id
                                       and c.ambit = a.id
                                       and c.genere = g.id
                                       and c.modalitat = m.id)
              order by e.vp)
     where rownum < 6
     order by vp desc;

  r_top5_aler c_top5_aler%rowtype;

  -- Cursor que retorna els 5 millors alers-pivot donats un mbit, un gnere i una modalitat.
  cursor c_top5_alerp is
    select distinct nom, vp
      from (select distinct j.nom, e.vp
              from partits p, jugadors j, estadistiques e
             where j.posicio = 4 -- codi d'aler-pivot
               and e.partit = p.id
               and e.jugador = j.id
               and p.competicio in (select distinct c.id
                                      from competicions c, ambits a, generes g, modalitats m
                                     where c.ambit = p_ambit_id
                                       and c.genere = p_genere_id
                                       and c.modalitat = p_modalitat_id
                                       and c.ambit = a.id
                                       and c.genere = g.id
                                       and c.modalitat = m.id)
              order by e.vp)
     where rownum < 6
     order by vp desc;

  r_top5_alerp c_top5_alerp%rowtype;

  -- Cursor que retorna els 5 millors pivots donats un mbit, un gnere i una modalitat.
  cursor c_top5_pivot is
    select distinct nom, vp
      from (select distinct j.nom, e.vp
              from partits p, jugadors j, estadistiques e
             where j.posicio = 5 -- codi de pivot
               and e.partit = p.id
               and e.jugador = j.id
               and p.competicio in (select distinct c.id
                                      from competicions c, ambits a, generes g, modalitats m
                                     where c.ambit = p_ambit_id
                                       and c.genere = p_genere_id
                                       and c.modalitat = p_modalitat_id
                                       and c.ambit = a.id
                                       and c.genere = g.id
                                       and c.modalitat = m.id)
              order by e.vp)
     where rownum < 6
     order by vp desc;

  r_top5_pivot c_top5_pivot%rowtype;

  begin
    -- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_TOP5_JUGADORS','PAR1: '||p_ambit_id||
                                                               ' - PAR2: '||p_genere_id||
                                                               ' - PAR3: '||p_modalitat_id);
	  if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment, ja que el procediment s de consulta.
	    pkg_control.prc_exec_test(v_id);
    end if;

    -- Augmentem el tamany del buffer de l'output suficient perqu cpiga tot el missatge.
    dbms_output.enable(1000000);
    -- Assigna a la variable v_ambit el nom de l'mbit corresponent amb el id passat com a parmetre:
    open c_ambit;
      fetch c_ambit into v_ambit;
    close c_ambit;

    -- Assigna a la variable v_genere el nom del gnere corresponent amb el id passat com a parmetre:
    open c_genere;
      fetch c_genere into v_genere;
    close c_genere;

    -- Assigna a la variable v_modalitat el nom de la modalitat corresponent amb el id passat com a parmetre.
    open c_modalitat;
      fetch c_modalitat into v_modalitat;
    close c_modalitat;

    dbms_output.put_line('+++++++++++++++++++++++++++');
    dbms_output.put_line('TOP 5 DELS MILLORS JUGADORS');
    dbms_output.put_line('MBIT: '||v_ambit);
    dbms_output.put_line('GNERE: '||v_genere);
    dbms_output.put_line('MODALITAT: '||v_modalitat);
    dbms_output.put_line('+++++++++++++++++++++++++++');
    dbms_output.new_line;
    dbms_output.new_line;

    dbms_output.put_line('+++++++++++++');
    dbms_output.put_line('POSICIO: BASE');
    dbms_output.put_line('+++++++++++++');
    dbms_output.new_line;
		-- S'obre el cursor amb les dades del top base.
		for i in c_top5_base loop
      dbms_output.put_line('JUGADOR: '||i.nom);
			dbms_output.put_line('VALOPRACI PONDERADA: '||i.vp);
      dbms_output.new_line;
		end loop;

    dbms_output.put_line('++++++++++++++++');
    dbms_output.put_line('POSICIO: ESCORTA');
    dbms_output.put_line('++++++++++++++++');
    dbms_output.new_line;
		-- S'obre el cursor amb les dades del top escorta.
		for i in c_top5_escorta loop
      dbms_output.put_line('JUGADOR: '||i.nom);
			dbms_output.put_line('VALOPRACI PONDERADA: '||i.vp);
      dbms_output.new_line;
		end loop;

    dbms_output.put_line('+++++++++++++');
    dbms_output.put_line('POSICIO: ALER');
    dbms_output.put_line('+++++++++++++');
    dbms_output.new_line;
		-- S'obre el cursor amb les dades del top aler.
		for i in c_top5_aler loop
      dbms_output.put_line('JUGADOR: '||i.nom);
			dbms_output.put_line('VALOPRACI PONDERADA: '||i.vp);
      dbms_output.new_line;
		end loop;

    dbms_output.put_line('+++++++++++++++++++');
    dbms_output.put_line('POSICIO: ALER-PIVOT');
    dbms_output.put_line('+++++++++++++++++++');
    dbms_output.new_line;
		-- S'obre el cursor amb les dades del top aler-pivot.
		for i in c_top5_alerp loop
      dbms_output.put_line('JUGADOR: '||i.nom);
			dbms_output.put_line('VALOPRACI PONDERADA: '||i.vp);
      dbms_output.new_line;
		end loop;
    
    dbms_output.put_line('++++++++++++++');
    dbms_output.put_line('POSICIO: PIVOT');
    dbms_output.put_line('++++++++++++++');
    dbms_output.new_line;
		-- S'obre el cursor amb les dades del top pivot.
		for i in c_top5_pivot loop
      dbms_output.put_line('JUGADOR: '||i.nom);
			dbms_output.put_line('VALOPRACI PONDERADA: '||i.vp);
      dbms_output.new_line;
		end loop;

		-- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		pkg_control.prc_exec_fi(v_id);
  exception
	  -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
    -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
    when others then
	    pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_top5_jugadors;


  ---------------------------------
  -- PRC_CONTRACTES_REPRESENTANT --
  ---------------------------------
  procedure prc_contractes_representant (
    p_any varchar2,
    p_representant_id varchar2
  ) is

  -- Variable que s'utilitza per emmagatzemar l'identificador nic del log que retorna la funci FUN_REG_EXEC.
  v_id number;
  v_nom_repre varchar2(100);

  --Cursor que retorna el nom del representant es correspon amb el id introduit com a parmetre.
  cursor c_nom_repre is
    select distinct nom
      from representants_jugadors
     where id = p_representant_id;

  -- Cursor que retorna el nombre de contractes i el valor econmic total de cadascun signats pel representant i any introduits com a parmetre.
  cursor c_cont_repre is
    select c.data_inici, c.data_fi, j.nom jugador, e1.nom equip_comprador, e2.nom equip_venedor, c.durada, c.salari_brut,
           c.compensacio_equip, c.total_operacio, m.descripcio moneda
      from contractes c, jugadors_representants jr, jugadors j, equips e1, equips e2, monedes m
     where extract(year from c.data_inici) = p_any
       and jr.representant = p_representant_id
       and c.jugador = jr.jugador
       and c.jugador = j.id
       and c.equip_comprador = e1.id
       and c.equip_venedor = e2.id
       and c.moneda = m.id
     order by c.data_inici;

  -- Cursor amb el nombre total i valor econmic dels contractes signats pel representant i any introduits com a parmetre.
  cursor c_cont_repre_tot is
    select count(*) num_contractes, sum(c.total_operacio) valor_total, c.moneda
      from contractes c, jugadors_representants jr
     where extract(year from c.data_inici) = p_any
       and jr.representant = p_representant_id
       and c.jugador = jr.jugador
    group by c.moneda;

  r_cont_repre_tot c_cont_repre_tot%rowtype;

  begin
    -- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_CONTRACTES_REPRESENTANT','PAR1: '||p_any||
                                                                        ' - PAR2: '||p_representant_id);
	  if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment, ja que el procediment s de consulta.
	    pkg_control.prc_exec_test(v_id);
    end if;

    -- Augmentem el tamany del buffer de l'output suficient perqu cpiga tot el missatge.
    dbms_output.enable(1000000);
    -- Assigna a la variable v_nom_repre el nom del representant amb el id passat com a parmetre.
    open c_nom_repre;
    fetch c_nom_repre into v_nom_repre;
    close c_nom_repre;

    dbms_output.put_line('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.put_line('LLISTAT DELS CONTRACTES DEL REPRESENTANT: '||p_representant_id||' A L''ANY: '||p_any);
    dbms_output.put_line('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.new_line;
    dbms_output.new_line;

		-- S'obre el cursor amb les dades dels contractes signats en l'any introduit per parmetre.
		for i in c_cont_repre loop
      dbms_output.put_line('DATA INICI: '||i.data_inici);
			dbms_output.put_line('DATA FI: '||i.data_fi);
      dbms_output.put_line('JUGADOR: '||i.jugador);
      dbms_output.put_line('EQUIP COMPRADOR: '||i.equip_comprador);
      dbms_output.put_line('EQUIP VENEDOR: '||i.equip_venedor);
      dbms_output.put_line('DURADA: '||i.durada||' dies');
      dbms_output.put_line('SALARI BRUT: '||to_char(i.salari_brut,'999G999G990D00')||' '||i.moneda);
      dbms_output.put_line('COMPENSACI EQUIP: '||to_char(i.compensacio_equip,'999G999G990D00')||' '||i.moneda);
      dbms_output.put_line('TOTAL OPERACI: '||to_char(i.total_operacio,'999G999G990D00')||' '||i.moneda);
      dbms_output.new_line;
    end loop;

    -- Obre el cursor amb els representants dels jugadors.
    open c_cont_repre_tot;
      fetch c_cont_repre_tot into r_cont_repre_tot;
    close c_cont_repre_tot;

    dbms_output.put_line('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.put_line('NMERO TOTAL DE CONTRACTES SIGNATS: '||nvl(r_cont_repre_tot.num_contractes,0));
    dbms_output.put_line('VALOR TOTAL DE TOTS ELS CONTRACTES: '||to_char(nvl(r_cont_repre_tot.valor_total,0),'999G999G990D00')||' '||r_cont_repre_tot.moneda);
    dbms_output.put_line('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.new_line;
    -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		pkg_control.prc_exec_fi(v_id);
  exception
	  -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
    -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
    when others then
	    pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_contractes_representant;


  -----------------------------
  -- PRC_TOP10_EQUIPS_GASTAT --
  -----------------------------
  procedure prc_top10_equips_gastat (
    p_any varchar2
  ) is

  -- Variable que s'utilitza per emmagatzemar l'identificador nic del log que retorna la funci FUN_REG_EXEC.
  v_id number;

  -- Cursor que retorna el nombre de contractes i el valor econmic total de cadascun signats pel representant i any introduits com a parmetre.
  cursor c_top10 is
    select distinct equip, total_gastat, moneda
      from (select e.nom equip, sum(c.total_operacio) total_gastat, c.moneda
              from contractes c, equips e
             where extract(year from c.data_inici) = p_any
               and c.equip_comprador = e.id
             group by e.nom, c.moneda
             order by sum(c.total_operacio) desc)
      where rownum < 11;

  begin
    -- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_TOP10_EQUIPS_GASTAT','PAR1: '||p_any);
	  if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment, ja que el procediment s de consulta.
	    pkg_control.prc_exec_test(v_id);
    end if;

    -- Augmentem el tamany del buffer de l'output suficient perqu cpiga tot el missatge.
    dbms_output.enable(1000000);

    dbms_output.put_line('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.put_line('TOP 10 EQUIPS QUE MS HAN GASTAT EN FITXATGES A L''ANY:'||p_any);
    dbms_output.put_line('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.new_line;

		-- S'obre el cursor amb les dades dels contractes signats en l'any introduit per parmetre.
		for i in c_top10 loop
      dbms_output.put_line('EQUIP: '||i.equip);
			dbms_output.put_line('IMPORTAT GASTAT: '||to_char(i.total_gastat,'999G999G990D00')||' '||i.moneda);
      dbms_output.new_line;
    end loop;

    -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		pkg_control.prc_exec_fi(v_id);
  exception
	  -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
    -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
    when others then
	    pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_top10_equips_gastat;


  --------------------------
  -- PRC_JUGADORS_LLIURES --
  --------------------------
  procedure prc_jugadors_lliures (
    p_pais_id varchar2,
    p_genere_id varchar2,
    p_modalitat_id varchar2
  ) is

  -- Variable que s'utilitza per emmagatzemar l'identificador nic del log que retorna la funci FUN_REG_EXEC.
  v_id number;
  v_pais varchar2(50);
  v_genere varchar2(20);
  v_modalitat varchar2(40);

  -- Cursor que retorna el nom pas que es correspon al id passat per parmetre.
  cursor c_pais is
    select distinct descripcio pais
      from paisos_continents
     where id = p_pais_id;

  -- Cursor que retorna el gnere que es correspon al id passat per parmetre.
  cursor c_genere is
    select distinct descripcio genere
      from generes
     where id = p_genere_id;

  -- Cursor que retorna la modalitat que es correspon al id passat per parmetre.
  cursor c_modalitat is
    select distinct descripcio modalitat
      from modalitats
     where id = p_modalitat_id;

  cursor c_sense_equip is
    select distinct j.nom, p.descripcio posicio
      from jugadors j, posicions p
     where j.posicio = p.id
       and j.estat = 'A'
       and j.federacio = p_pais_id
       and not exists (select 'x' from jugadors_equips jr
                        where jr.jugador = j.id);

  cursor c_fi_contr is
    select distinct j.nom,
           last_value(c.data_fi) over (partition by c.jugador order by c.data_fi) data_fi
      from jugadors j, contractes c, partits p, competicions co, jugadors_partits jp, paisos_continents pc
     where pc.id = p_pais_id
       and j.federacio = pc.id
       and co.genere = p_genere_id
       and co.modalitat = p_modalitat_id
       and extract (year from sysdate) = (case when length(co.temporada) > 4 then substr(co.temporada,6) else co.temporada end) -- ltima temporada
       and c.jugador = j.id
       and p.competicio = co.id
       and jp.jugador = j.id
       and jp.partit = p.id
       and (select last_value(extract (year from con.data_fi)) over (partition by con.jugador order by con.data_fi)
              from contractes con
             where con.jugador = j.id) = extract (year from sysdate); -- acaben contracte l'any en curs

  begin
    -- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_TOP10_EQUIPS_GASTAT','PAR1: '||p_pais_id||
                                                                    ' - PAR2: '||p_genere_id||
                                                                    ' - PAR3: '||p_modalitat_id);
	  if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment, ja que el procediment s de consulta.
	    pkg_control.prc_exec_test(v_id);
    end if;

    -- Assigna a la variable v_pais el nom del pas corresponent amb el id passat com a parmetre:
    open c_pais;
      fetch c_pais into v_pais;
    close c_pais;

    -- Assigna a la variable v_genere el nom del gnere corresponent amb el id passat com a parmetre:
    open c_genere;
      fetch c_genere into v_genere;
    close c_genere;

    -- Assigna a la variable v_modalitat el nom de la modalitat corresponent amb el id passat com a parmetre:
    open c_modalitat;
      fetch c_modalitat into v_modalitat;
    close c_modalitat;


    -- Augmentem el tamany del buffer de l'output suficient perqu cpiga tot el missatge.
    dbms_output.enable(1000000);

    dbms_output.put_line('+++++++++++++++++++++++++++++++++++++++++');
    dbms_output.put_line('LLISTAT DE JUGADORS PER POSIBLES FIXATGES');
    dbms_output.put_line('PAS: '||v_pais);
    dbms_output.put_line('GNERE: '||v_genere);
    dbms_output.put_line('MODALITAT: '||v_modalitat);
    dbms_output.put_line('+++++++++++++++++++++++++++++++++++++++++');
    dbms_output.new_line;

    dbms_output.put_line('++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.put_line('JUGADORS QUE FINALITZEN CONTRACTE LA TEMPORADA EN CURS');
    dbms_output.put_line('++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.new_line;

    for i in c_fi_contr loop
      dbms_output.put_line('NOM: '||i.nom);
      dbms_output.put_line('DATA FI DE CONTRACTE: '||to_char(i.data_fi,'dd/mm/yyyy'));
      dbms_output.new_line;
    end loop;
    
    dbms_output.put_line('++++++++++++++++++++');
    dbms_output.put_line('JUGADORS SENSE EQUIP');
    dbms_output.put_line('++++++++++++++++++++');
    dbms_output.new_line;

    for i in c_sense_equip loop
      dbms_output.put_line('NOM: '||i.nom);
      dbms_output.put_line('POSICI: '||i.posicio);
      dbms_output.new_line;
    end loop;

    -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		pkg_control.prc_exec_fi(v_id);
  exception
	  -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
    -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
    when others then
	    pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_jugadors_lliures;

end pkg_consultes;
/

spool off;
