/*
	NOM PAQUET: PKG_CONTRACTE
	
	FUNCIONALITAT: Permet actualizar les dades al mdul de datawarehouse i retorna les mides precalculades.

*/

spool 4_6_Creacio_Pkg_Dwh.log;
  
----------------
-- CAPAALERA --
----------------
create or replace
package pkg_dwh is

  -------------
  -- PKG_DWH --
  -------------

	-- Variable que cont el nom del paquet que s'utilitza per a registar-lo a la taula LOG_PROC_EXEC.
	pck_nom varchar2(7):='PKG_DWH';

	-- Variable que s'utilitza per determinar si el sistema es troba en mode TEST.
	v_test_actiu boolean;

	-- Inicialitza la dimensi DIM_DATA dins el periode que se li passa per parmetre.
	procedure prc_ini_dim_data (p_data_ini date, p_data_fi date);

  -- Actualitza les dades de la taula CONTROL_DWH, per tal de saber quan s'han actualitzat per darrera vegada les dimensions i les taules de fets.
  procedure prc_act_control_dwh (p_proc varchar2, p_data_ini date, p_data_fi date);

  -- Insereix un registre amb el id d'un jugador a la dimensi DIM_JUGADOR.
  procedure prc_dim_jugador (p_jugador_id varchar2);

  -- Insereix un registre amb el id d'un equip a la dimensi DIM_EQUIP.
  procedure prc_dim_equip (p_equip_id varchar2);

  -- Insereix un registre amb el id d'una competici a la dimensi DIM_COMPETICIO.
  procedure prc_dim_competicio (p_competicio_id number);

  -- Insereix un registre amb el id d'una modalitat a la dimensi DIM_MODALITAT.
  procedure prc_dim_modalitat (p_modalitat_id varchar2);

  -- Insereix un registre amb el id d'un gnere a la dimensi DIM_GENERE.
  procedure prc_dim_genere (p_genere_id varchar2);

  -- Insereix un registre amb el id d'un pas a la dimensi DIM_PAIS.
  procedure prc_dim_pais (p_pais_id varchar2);

  -- Carrega les dades a la dimensi DIM_JUGADOR.
  procedure prc_ini_dim_jugador (p_data_ini date, p_data_fi date);

  -- Carrega les dades a la dimensi DIM_EQUIP.
  procedure prc_ini_dim_equip (p_data_ini date, p_data_fi date);

  -- Carrega les dades a la dimensi DIM_COMPETICIO.
  procedure prc_ini_dim_competicio (p_data_ini date, p_data_fi date);

  -- Carrega les dades a la dimensi DIM_MODALITAT.
  procedure prc_ini_dim_modalitat (p_data_ini date, p_data_fi date);

  -- Carrega les dades a la dimensi DIM_GENERE.
  procedure prc_ini_dim_genere (p_data_ini date, p_data_fi date);

  -- Carrega les dades a la dimensi DIM_PAIS.
  procedure prc_ini_dim_pais (p_data_ini date, p_data_fi date);

  -- Insereix a la taula de fets JUGADORS_ACTIUS un registre amb el id del gnere, el id de la modalitat
  -- i el nombre de jugadors actius d'aquests gnere i modalitat.
  procedure prc_jug_actius (p_data_ini date, p_data_fi date);

	-- Insereix a la taula de fets MAXIM_ANOTADOR un registre amb el id de la competicio, el id de la data,
  -- el id del jugador i el nmero de punts encistellats del jugador d'aquesta competici i data.
  procedure prc_max_anotador (p_data_ini date, p_data_fi date);

  -- Insereix a la taula de fets JUGADOR_MES_BEN_PAGAT un registre amb el id de modalitat, el id del gnere,
  -- el id del jugador i el sou del jugador en aquesta modalitat i gnere.
  procedure prc_jug_mes_ben_pagat (p_data_ini date, p_data_fi date);

  -- Insereix a la taula de fets JUGADOR_MES_GUANYS el id del jugador i els seus guanys.
  procedure prc_jug_mes_guanys (p_data_ini date, p_data_fi date);

	-- Insereix a la taula de fets RANKING_SOU_MIG el id del pas, el id de la data, el id del gnere, el id de la modalitat
  -- i el sou mig d'aquest pas, data, gnere i modalitat.
  procedure prc_sou_mig (p_data_ini date, p_data_fi date);

  -- Insereix la taula de fets MILLORS_EQUIPS el id de la competici, el id de la data, el id de l'equip
  -- i els punts encistellas i rebuts de l'equip en aquesta competici i data.
  procedure prc_millors_equips (p_data_ini date, p_data_fi date);

  -- Insereix a la taula de fets MILLOR_JUGADOR el id del gnere, el id de la modalitat, el id del jugador i la valoraci ponderada del jugador
  -- en aquest gnere i modalitat el darreny any.
  procedure prc_millor_jugador (p_data_ini date, p_data_fi date);

	-- Per al rang de dates introduides, carrega totes les dimensions: DIM_DATA, DIM_EQUIP, DIM_GENERE, DIM_JUGADOR, DIM_MODALITAT i DIM_PAIS,
  -- i les taules de fets: JUGADORS_ACTIUS, MAXIM_ANOTADOR, JUGADOR_MES_BEN_PAGAT, JUGADOR_MES_GUANYS, RANKING_SOU_MIG, MILLORS_EQUIPS i MILLOR_JUGADOR.
	procedure prc_act_dwh (p_data_ini date, p_data_fi date);

  -- Retorna el nombre de jugadors actius de tots els gneres i modalitats.
  procedure res_jug_actius;

  -- Retorna el mxim anotador de la competici amb el id passat per parmetre al procediment.
  procedure res_maxim_anotador (p_competicio_id varchar2);

  -- Retorna el jugador ms ben pagat de cada modalitat i gnere.
  procedure res_jug_mes_ben_pagat;

  -- Retorn el jugador amb ms guanys de la histria.
  procedure res_jug_mes_guanys;

  -- Retorna el sou mig anual dels jugadors d'un pas, temporada, gnere i modalitat.
  procedure res_sou_mig (p_pais varchar2, p_temporada varchar2, p_genere varchar2, p_modalitat varchar2);

  -- Retorna els millors equips ofensius i defensius d'una competici i temporada.
  procedure res_millors_equips (p_competicio_id number, p_temporada varchar2);

  -- Retorna el millor jugador del mn de cada gnere i modalitat del darrer any.
  procedure res_millor_jugador;

end pkg_dwh;
/

--------------------
-- COS DEL PAQUET --
--------------------
create or replace
package body pkg_dwh is

	-----------------------
	--  PRC_INI_DIM_DATA --
	-----------------------
	procedure prc_ini_dim_data (
    p_data_ini date,
    p_data_fi date
  ) is

	-- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

  v_data_ini_num number;
  v_data_fi_num number;
  v_cont number;
  r_ddata dim_data%rowtype;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_INI_DIM_DATA','PAR1: '||p_data_ini||
                                                             ' - PAR2: '||p_data_fi);

		-- Registra a la taula CONTROL_DWH que s'executar el procediment per a un rang de dates concretes.
		pkg_dwh.prc_act_control_dwh('PRC_INI_DIM_DATA',p_data_ini,p_data_fi);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		else
      -- Convertim les dates d'inici i fi en dos numrics amb el format "aaaammdd".
      v_data_ini_num:=to_number(to_char(p_data_ini,'yyyymmdd'));
      v_data_fi_num:=to_number(to_char(p_data_fi,'yyyymmdd'));

      loop
        r_ddata.id_data    :=   v_data_ini_num;
        r_ddata.data_any   :=   to_char(substr(v_data_ini_num,1,4));
        r_ddata.data_mes   :=   to_char(substr(v_data_ini_num,5,2));
        r_ddata.data_dia   :=   to_char(substr(v_data_ini_num,7));

        -- Insereix a la dimensi DIM_DATA un registre amb l'identificador de data, any, mes i dia.
        insert into dim_data values r_ddata;
        v_data_ini_num:=v_data_ini_num + 1;
        if v_data_ini_num > v_data_fi_num then
          exit;
        end if;
      end loop;
    end if;
    -- Confirma la inserci a la dimensi DIM_DATA.
    commit;
		 -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament insereix el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));
  end prc_ini_dim_data;


  -------------------------
  -- PRC_ACT_CONTROL_DWH --
  -------------------------
  procedure prc_act_control_dwh (
    p_proc varchar2,
    p_data_ini date,
    p_data_fi date
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_ACT_CONTROL_DWH','PAR1: '||p_proc||
                                                                ' - PAR2: '||p_data_ini||
                                                                ' - PAR3: '||p_data_fi);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		else
      -- Si no es troba en mode TEST, seguim l'execuci normal i actualitzem el registre corresponent de la taula CONTROL_DWH.
			update control_dwh set data_inici_darr_exec   =   p_data_ini,
                             data_fi_darr_exec      =   p_data_fi,
                             data_exec              =   sysdate
       where procediment = p_proc;
    end if;
    -- Confirma l'actualitzaci realitzada a la taula CONTROL_DWH.
    commit;
		 -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament insereix el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_act_control_dwh;


  ---------------------
  -- PRC_DIM_JUGADOR --
  ---------------------
  procedure prc_dim_jugador (
    p_jugador_id varchar2
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_DIM_JUGADOR','PAR1: '||p_jugador_id);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal per no s'insereix cap registre a la dimensi DIM_JUGADOR.
		  pkg_control.prc_exec_test(v_id);
		else
      -- Insereix a la dimensi DIM_JUGADOR un registre amb l'identificador del jugador.
      insert into dim_jugador values (p_jugador_id);
    end if;
    -- Confirma la inserci a la dimensi DIM_JUGADOR.
    commit;
		 -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_dim_jugador;


  -----------------------
  -- PRC_DIM_EQUIP --
  -----------------------
  procedure prc_dim_equip (
    p_equip_id varchar2
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_DIM_EQUIP','PAR1: '||p_equip_id);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal per no s'insereix cap registre a la dimensi DIM_EQUIP.
		  pkg_control.prc_exec_test(v_id);
		else
      -- Insereix a la dimensi DIM_EQUIP un registre amb l'identificador de l'equip.
      insert into dim_equip values (p_equip_id);
    end if;
    -- Confirma la inserci a la dimensi DIM_EQUIP.
    commit;
		 -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_dim_equip;


  ----------------------------
  -- PRC_DIM_COMPETICIO --
  ----------------------------
  procedure prc_dim_competicio (
   p_competicio_id number
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

	begin
		-- RegiStra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_DIM_COMPETICIO','PAR1: '||p_competicio_id);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal per no s'insereix cap registre a la dimensi DIM_COMPETICIO.
		  pkg_control.prc_exec_test(v_id);
		else
      -- Insereix a la dimensi DIM_COMPETICIO un registre amb l'identificador de la competici.
      insert into dim_competicio values (p_competicio_id);
    end if;
    -- Confirma la inserci a la dimensi DIM_COMPETCIO.
    commit;
		 -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_dim_competicio;


  ---------------------------
  -- PRC_DIM_MODALITAT --
  ---------------------------
  procedure prc_dim_modalitat (
    p_modalitat_id varchar2
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_DIM_MODALITAT','PAR1: '||p_modalitat_id);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal per no s'insereix cap registre a la dimensi DIM_MODALITAT.
		  pkg_control.prc_exec_test(v_id);
		else
      -- Insereix a la dimensi DIM_MODALITAT un registre amb l'identificador de la modalitat.
      insert into dim_modalitat values (p_modalitat_id);
    end if;
    -- Confirma la inserci a la dimensi DIM_MODALITAT.
    commit;
		 -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_dim_modalitat;


  --------------------
  -- PRC_DIM_GENERE --
  --------------------
  procedure prc_dim_genere (
    p_genere_id varchar2
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_DIM_GENERE','PAR1: '||p_genere_id);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal per no s'insereix cap registre a la dimensi DIM_GENERE.
		  pkg_control.prc_exec_test(v_id);
		else
      -- Insereix a la dimensi DIM_GENERE un registre amb l'identificador del gnere.
      insert into dim_genere values (p_genere_id);
    end if;
    -- Confirma la inserci a la dimensi DIM_GENERE.
    commit;
		 -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_dim_genere;


  ------------------
  -- PRC_DIM_PAIS --
  ------------------
  procedure prc_dim_pais (
    p_pais_id varchar2
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_DIM_PAIS','PAR1: '||p_pais_id);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal per no s'insereix cap registre a la dimensi DIM_PAIS.
		  pkg_control.prc_exec_test(v_id);
		else
      -- Insereix a la dimensi DIM_PAIS un registre amb l'identificador del pas.
      insert into dim_pais values (p_pais_id);
    end if;
    -- Confirma la inserci a la dimensi DIM_PAIS.
    commit;
		 -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_dim_pais;


  -------------------------
  -- PRC_INI_DIM_JUGADOR --
  -------------------------
  procedure prc_ini_dim_jugador (
    p_data_ini date,
    p_data_fi date
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

  cursor c_jug is
    select distinct id
      from jugadors;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_INI_DIM_JUGADOR','PAR1: '||p_data_ini||
                                                                ' - PAR2: '||p_data_fi);

    -- Registra a la taula CONTROL_DWH que s'executar el procediment per a un rang de dates concretes.
		pkg_dwh.prc_act_control_dwh('PRC_INI_DIM_JUGADOR',p_data_ini,p_data_fi);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		else
      -- -- Per cada equip executa el procs PRC_DIM_JUGADOR.
      for i in c_jug loop
        prc_dim_jugador (i.id);
      end loop;
    end if;
    -- Confirma les insercions realitzades a la dimensi DIM_JUGADOR.
    commit;
		 -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_ini_dim_jugador;


  -----------------------
  -- PRC_INI_DIM_EQUIP --
  -----------------------
  procedure prc_ini_dim_equip (
    p_data_ini date,
    p_data_fi date
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

  cursor c_equip is
    select distinct id
      from equips;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_INI_DIM_EQUIP','PAR1: '||p_data_ini||
                                                              ' - PAR2: '||p_data_fi);

    -- Registra a la taula CONTROL_DWH que s'executar el procediment per a un rang de dates concretes.
		pkg_dwh.prc_act_control_dwh('PRC_INI_DIM_EQUIP',p_data_ini,p_data_fi);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		else
      -- Per cada equip executa el procs PRC_DIM_EQUIP.
      for i in c_equip loop
        prc_dim_equip (i.id);
      end loop;
    end if;
    -- Confirma les insercions realitzades a la dimensi DIM_EQUIP.
    commit;
		 -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_ini_dim_equip;


  ----------------------------
  -- PRC_INI_DIM_COMPETICIO --
  ----------------------------
  procedure prc_ini_dim_competicio (
    p_data_ini date,
    p_data_fi date
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

  cursor c_comp is
    select distinct id
      from competicions
     where (case when length(temporada) > 4 then substr(temporada,6) else temporada end) between to_char(p_data_ini,'yyyy') and to_char(p_data_fi,'yyyy');

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_INI_DIM_COMPETICIO','PAR1: '||p_data_ini||
                                                                   ' - PAR2: '||p_data_fi);

    -- Registra a la taula CONTROL_DWH que s'executar el procediment per a un rang de dates concretes.
		pkg_dwh.prc_act_control_dwh('PRC_INI_DIM_COMPETICIO',p_data_ini,p_data_fi);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		else
      -- -- Per cada equip executa el procs PRC_DIM_COMPETICIO.
      for i in c_comp loop
        prc_dim_competicio (i.id);
      end loop;
    end if;
    -- Confirma les insercions a la dimensi DIM_COMPETICIO.
    commit;
		 -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_ini_dim_competicio;


  ---------------------------
  -- PRC_INI_DIM_MODALITAT --
  ---------------------------
  procedure prc_ini_dim_modalitat (
    p_data_ini date,
    p_data_fi date
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

  cursor c_mod is
    select distinct id
      from modalitats;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_INI_DIM_MODALITAT','PAR1: '||p_data_ini||
                                                                  ' - PAR2: '||p_data_fi);

    -- Registra a la taula CONTROL_DWH que s'executar el procediment per a un rang de dates concretes.
		pkg_dwh.prc_act_control_dwh('PRC_INI_DIM_MODALITAT',p_data_ini,p_data_fi);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		else
      -- Insereix a la dimensi DIM_PAIS un registre amb l'identificador del pas.
      for i in c_mod loop
        insert into dim_modalitat values (i.id);
      end loop;
    end if;
    -- Confirma la inserci a la dimensi DIM_MODALITAT.
    commit;
		 -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_ini_dim_modalitat;


  ------------------------
  -- PRC_INI_DIM_GENERE --
  ------------------------
  procedure prc_ini_dim_genere (
    p_data_ini date,
    p_data_fi date
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

  cursor c_gen is
    select distinct id
      from generes;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_INI_DIM_GENERE','PAR1: '||p_data_ini||
                                                               ' - PAR2: '||p_data_fi);

    -- Registra a la taula CONTROL_DWH que s'executar el procediment per a un rang de dates concretes.
		pkg_dwh.prc_act_control_dwh('PRC_INI_DIM_GENERE',p_data_ini,p_data_fi);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		else
      -- Insereix a la dimensi DIM_PAIS un registre amb l'identificador del pas.
      for i in c_gen loop
        insert into dim_genere values (i.id);
      end loop;
    end if;
    -- Confirma les insercions a la dimensi DIM_GENERE.
    commit;
		 -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_ini_dim_genere;


  ----------------------
  -- PRC_INI_DIM_PAIS --
  ----------------------
  procedure prc_ini_dim_pais (
    p_data_ini date,
    p_data_fi date
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

  cursor c_pais is
    select distinct id
      from paisos_continents;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_INI_DIM_PAIS','PAR1: '||p_data_ini||
                                                             ' - PAR2: '||p_data_fi);

    -- Registra a la taula CONTROL_DWH que s'executar el procediment per a un rang de dates concretes.
		pkg_dwh.prc_act_control_dwh('PRC_INI_DIM_PAIS',p_data_ini,p_data_fi);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		else
      -- Insereix a la dimensi DIM_PAIS un registre amb l'identificador del pas.
      for i in c_pais loop
        insert into dim_pais values (i.id);
      end loop;
    end if;
    -- Confirma les insercions a la dimensi DIM_PAIS.
    commit;
		 -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_ini_dim_pais;


  --------------------
  -- PRC_JUG_ACTIUS --
  --------------------
  procedure prc_jug_actius (
    p_data_ini date,
    p_data_fi date
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

  cursor c_gen is
    select distinct id_genere
      from dim_genere;

  cursor c_mod (p_gen varchar2) is
    select distinct m.id_modalitat
      from dim_modalitat m, competicions c
     where c.genere = p_gen
       and c.modalitat = m.id_modalitat;

  cursor c_jug_actius (p_gen varchar2, p_mod varchar2) is
    select distinct count(distinct j.id) num_jugadors
      from jugadors j, competicions c, partits p, jugadors_partits jp
     where c.genere = p_gen
       and c.modalitat = p_mod
       and c.id = p.competicio
       and jp.jugador = j.id
       and jp.partit = p.id
       and j.estat = 'A'
       and p.data between p_data_ini and p_data_fi;  -- ACTIU

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_JUG_ACTIUS','PAR1: '||p_data_ini||
                                                           ' - PAR2: '||p_data_fi);

    -- Registra a la taula CONTROL_DWH que s'executar el procediment per a un rang de dates concretes.
		pkg_dwh.prc_act_control_dwh('PRC_JUG_ACTIUS',p_data_ini,p_data_fi);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		else
      -- Per cada gnere i modalitat insereix un registre a la taula de dets JUGADORS_ACTIUS amb el gnere, modalitat i nmero de jugadors.
      for i in c_gen loop
        for j in c_mod (i.id_genere) loop
          for k in c_jug_actius (i.id_genere,j.id_modalitat) loop
            insert into jugadors_actius values (i.id_genere,j.id_modalitat,k.num_jugadors);
          end loop;
        end loop;
      end loop;
    end if;
    -- Confirma la inserci a la taula de fets JUGADORS_ACTIUS.
    commit;
		 -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_jug_actius;


  ----------------------
  -- PRC_MAX_ANOTADOR --
  ----------------------
  procedure prc_max_anotador (
    p_data_ini date,
    p_data_fi date
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;
  v_data_ini_num number;
  v_data_fi_num number;

  cursor c_comp is
    select distinct id_competicio
      from dim_competicio
     order by id_competicio;

  cursor c_max_anotador (p_comp number) is
    select distinct id_competicio, id_jugador, punts
      from (select distinct c.id id_competicio, es.jugador id_jugador, sum(es.pt) punts
              from competicions c, partits p, estadistiques es
             where c.id = p_comp
               and c.id = p.competicio
               and p.id = es.partit
               and p.data between p_data_ini and p_data_fi
             group by c.id, es.jugador
             order by sum(es.pt) desc)
     where rownum < 2;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_MAX_ANOTADOR','PAR1: '||p_data_ini||
                                                             ' - PAR2: '||p_data_fi);

    -- Registra a la taula CONTROL_DWH que s'executar el procediment per a un rang de dates concretes.
		pkg_dwh.prc_act_control_dwh('PRC_MAX_ANOTADOR',p_data_ini,p_data_fi);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		else
      v_data_ini_num:=to_number(to_char(p_data_ini,'yyyymmdd'));
      -- Per cada id de competici insereix a la taula de fets MAXIM_ANOTADOR un registre amb el id de la competicio,
      -- el id de la data, el id del jugador que s el mxim a notador i el nombre de punts anotats.
      for i in c_comp loop
        for j in c_max_anotador (i.id_competicio) loop
          insert into maxim_anotador values (i.id_competicio,v_data_ini_num,j.id_jugador,j.punts);
        end loop;
      end loop;
    end if;
    -- Confirma la inserci a la taula de fets MAXIM_ANOTADOR.
    commit;
		 -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_max_anotador;


  ---------------------------
  -- PRC_JUG_MES_BEN_PAGAT --
  ---------------------------
  procedure prc_jug_mes_ben_pagat (
    p_data_ini date,
    p_data_fi date
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

  cursor c_mod is
    select distinct id_modalitat
      from dim_modalitat;

  cursor c_gen (p_mod varchar2) is
    select distinct g.id_genere
      from dim_genere g, competicions c
     where c.modalitat = p_mod
       and c.genere = g.id_genere;

  cursor c_contr (p_mod varchar2, p_gen varchar2) is
    select * from (select distinct id_jugador, sou
                     from (select distinct c.jugador id_jugador,
                                           first_value (c.salari_brut) over (partition by c.jugador order by c.data_inici desc) sou
                                      from contractes c, competicions co, partits p, jugadors_partits jp
                                     where co.modalitat = p_mod
                                       and co.genere = p_gen
                                       and p.competicio = co.id
                                       and jp.partit = p.id
                                       and jp.jugador = c.jugador
                                       and c.data_inici between p_data_ini and p_data_fi)
                    order by sou desc)
     where rownum < 2;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_JUG_MES_BEN_PAGAT','PAR1: '||p_data_ini||
                                                                  ' - PAR2: '||p_data_fi);

    -- Registra a la taula CONTROL_DWH que s'executar el procediment per a un rang de dates concretes.
		pkg_dwh.prc_act_control_dwh('PRC_JUG_MES_BEN_PAGAT',p_data_ini,p_data_fi);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		else
      -- Per cada modalitat i gnere insereix a la taula de fets JUGADORS_MES_BEN_PAGAT un registre amb el id de modalita, gnere i JUGADOR,
      -- i el sou corresponent al jugador inserit.
      for i in c_mod loop
        for j in c_gen (i.id_modalitat) loop
          for k in c_contr (i.id_modalitat, j.id_genere) loop
            insert into jugador_mes_ben_pagat values (i.id_modalitat,j.id_genere,k.id_jugador,k.sou);
          end loop;
        end loop;
      end loop;
    end if;
    -- Confirma la inserci a la taula de fets JUGADOR_MES_BEN_PAGAT.
    commit;
		 -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_jug_mes_ben_pagat;


  ------------------------
  -- PRC_JUG_MES_GUANYS --
  ------------------------
  procedure prc_jug_mes_guanys (
    p_data_ini date,
    p_data_fi date
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

  cursor c_guanys is
    select * from (
      select * from (
        select distinct jugador id_jugador, sum(salari_brut) guanys
          from contractes
         group by jugador)
       order by guanys desc)
     where rownum < 2;

  r_guanys c_guanys%rowtype;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_JUG_MES_GUANYS','PAR1: '||p_data_ini||
                                                               ' - PAR2: '||p_data_fi);

    -- Registra a la taula CONTROL_DWH que s'executar el procediment per a un rang de dates concretes.
		pkg_dwh.prc_act_control_dwh('PRC_JUG_MES_GUANYS',p_data_ini,p_data_fi);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		else
      -- Insereix a la taula de fets JUGADOR_MES_GUANYS un registre amb l'identificador del jugador i els gunys totals del jugador.
      open c_guanys;
        fetch c_guanys into r_guanys;
      close c_guanys;
      insert into jugador_mes_guany values (r_guanys.id_jugador,r_guanys.guanys);
    end if;
    -- Confirma la inserci a la taula de fets JUGADOR_MES_GUANYS.
    commit;
		 -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_jug_mes_guanys;


  -----------------
  -- PRC_SOU_MIG --
  -----------------
  procedure prc_sou_mig (
    p_data_ini date,
    p_data_fi date
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;
  v_data_ini_num number;
  v_data_fi_num number;

  cursor c_pais is
    select distinct id_pais
      from dim_pais;

  cursor c_gen (p_pais varchar2) is
    select distinct g.id_genere
      from dim_genere g, competicions c
     where c.pais_continent = p_pais
       and c.genere = g.id_genere;

  cursor c_mod (p_pais varchar2, p_gen varchar2) is
    select distinct m.id_modalitat
      from dim_modalitat m, competicions c
     where c.pais_continent = p_pais
       and c.genere = p_gen
       and c.modalitat = m.id_modalitat;

  cursor c_sou_mig (p_pais varchar2, p_gen varchar2, p_mod varchar2) is
    select round((sou/jugadors),2) sou_mig
      from (select count(c.jugador) jugadors, sum(c.salari_brut) sou
              from contractes c, competicions co, partits p, jugadors_partits jp
             where co.pais_continent = p_pais
               and co.genere = p_gen
               and co.modalitat = p_mod
               and co.id = p.competicio
               and jp.jugador = c.jugador
               and jp.partit = p.id
               and c.data_inici between p_data_ini and p_data_fi);

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_SOU_MIG','PAR1: '||p_data_ini||
                                                        ' - PAR2: '||p_data_fi);

    -- Registra a la taula CONTROL_DWH que s'executar el procediment per a un rang de dates concretes.
		pkg_dwh.prc_act_control_dwh('PRC_SOU_MIG',p_data_ini,p_data_fi);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		else
      v_data_ini_num:=to_number(to_char(p_data_ini,'yyyymmdd'));
      -- Per cada pas, gnere i modalitat insereix un registre a la taula de fets RANKING_SOU_MIG amb el id del pas, data, gnere,
      -- modalitat i el sou mig.
      for i in c_pais loop
        for j in c_gen (i.id_pais) loop
          for k in c_mod (i.id_pais, j.id_genere) loop
            for l in c_sou_mig (i.id_pais, j.id_genere, k.id_modalitat) loop
              insert into ranking_sou_mig values (i.id_pais,v_data_ini_num,j.id_genere,k.id_modalitat,l.sou_mig);
            end loop;
          end loop;
        end loop;
      end loop;
    end if;
    -- Confirma la inserci a la taula de fets RANKING_SOU_MIG.
    commit;
		 -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_sou_mig;


 ------------------------
 -- PRC_MILLORS_EQUIPS --
 ------------------------
 procedure prc_millors_equips (
   p_data_ini date,
   p_data_fi date
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;
  v_data_ini_num number;
  v_data_fi_num number;

  cursor c_comp is
    select distinct id id_competicio
      from competicions;

  cursor c_punts_marcats (p_comp number) is
    select * from (
      select * from (
        select distinct id_equip, punts,
               (select sum(est.pt) from estadistiques est
                 where est.partit = partit_id)-punts punts_rebuts
          from (select distinct je.equip id_equip, p.id partit_id, sum(es.pt) punts
                  from competicions c, partits p, equips e, estadistiques es, jugadors_partits jp, jugadors_equips je
                 where c.id = p_comp
                   and c.id = p.competicio
                   and es.partit = p.id
                   and jp.partit = p.id
                   and es.jugador = jp.jugador
                   and es.jugador = je.jugador
                   and je.equip = e.id
                   and p.data between p_data_ini and p_data_fi
                 group by je.equip, p.id))
                 order by punts desc)
     where rownum < 2;

  r_punts_marcats c_punts_marcats%rowtype;

  cursor c_punts_rebuts (p_comp number) is
    select * from (
      select * from (
        select distinct id_equip, punts,
               (select sum(est.pt) from estadistiques est
                 where est.partit = partit_id)-punts punts_rebuts
          from (select distinct je.equip id_equip, p.id partit_id, sum(es.pt) punts
                  from competicions c, partits p, equips e, estadistiques es, jugadors_partits jp, jugadors_equips je
                 where c.id = p_comp
                   and c.id = p.competicio
                   and es.partit = p.id
                   and jp.partit = p.id
                   and es.jugador = jp.jugador
                   and es.jugador = je.jugador
                   and je.equip = e.id
                   and p.data between p_data_ini and p_data_fi
                 group by je.equip, p.id))
                 order by punts_rebuts desc)
     where rownum < 2;

  r_punts_rebuts c_punts_rebuts%rowtype;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_MILLORS_EQUIPS','PAR1: '||p_data_ini||
                                                               ' - PAR2: '||p_data_fi);

    -- Registra a la taula CONTROL_DWH que s'executar el procediment per a un rang de dates concretes.
		pkg_dwh.prc_act_control_dwh('PRC_MILLORS_EQUIPS',p_data_ini,p_data_fi);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		else
      v_data_ini_num:=to_number(to_char(p_data_ini,'yyyymmdd'));
      -- Per cada id de competici insereix un registre a la taula de fets MILLORS_EQUIPS amb el id de la competici, data, equip,
      -- i els punts marcats i rebuts.
      for i in c_comp loop
        -- Pel id de la competici introduit obrim el cursor que cont l'equip que ms punts ha marcat.
        for j in c_punts_marcats(i.id_competicio) loop
          -- Insereix l'equip que ms punts ha marcat, els punts rebuts es deixen a nul.
          insert into millors_equips values (i.id_competicio,v_data_ini_num,j.id_equip,j.punts,null);
        end loop;
        -- Pel id de la competici introduit obrim el cursor que cont l'equip que ms punts ha rebut.
        for j in c_punts_rebuts(i.id_competicio) loop
          -- Insereix l'equip que ms punts ha rebut, els punts marcats es deixen a nul.
          insert into millors_equips values (i.id_competicio,v_data_ini_num,j.id_equip,null,j.punts);
        end loop;
      end loop;
    end if;
    -- Confirma les insercions a la taula de fets MILLORS_EQUIPS.
    commit;
		 -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_millors_equips;


  ------------------------
  -- PRC_MILLOR_JUGADOR --
  ------------------------
  procedure prc_millor_jugador (
    p_data_ini date,
    p_data_fi date
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;
  v_data_ini_num number;
  v_data_fi_num number;

  cursor c_comp is
    select distinct id_competicio
      from dim_competicio;

  cursor c_gen (p_comp number) is
    select distinct g.id_genere
      from dim_genere g, competicions c
     where c.id = p_comp
       and c.genere = g.id_genere;

  cursor c_mod (p_comp number, p_gen varchar2) is
    select distinct m.id_modalitat
      from dim_modalitat m, competicions c
     where c.id = p_comp
       and c.genere = p_gen
       and c.modalitat = m.id_modalitat;

  cursor c_millor_jug (p_comp number, p_gen varchar2, p_mod varchar2) is
    select * from (select distinct es.jugador, sum(es.vp) vp
                     from competicions c, partits p, estadistiques es, jugadors_partits jp
                    where c.id = p_comp
                      and c.genere = p_gen
                      and c.modalitat = p_mod
                      and c.id = p.competicio
                      and es.partit = p.id
                      and es.partit = jp.partit
                      and es.jugador = jp.jugador
                    group by es.jugador
                    order by sum(es.vp))
     where rownum < 2;

  r_millor_jug c_millor_jug%rowtype;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.PRC_MILLOR_JUGADOR','PAR1: '||p_data_ini||
                                                               ' - PAR2: '||p_data_fi);

    -- Registra a la taula CONTROL_DWH que s'executar el procediment per a un rang de dates concretes.
		pkg_dwh.prc_act_control_dwh('PRC_MILLOR_JUGADOR',p_data_ini,p_data_fi);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		else
      -- Per cada competici, gnere i modalitat insereix a la taula de fets MILLOR_JUGADOR un registre amb el id del gnere, modalitat
      -- i jugador, i la valoraci ponderada del jugador.
      for i in c_comp loop
        for j in c_gen (i.id_competicio) loop
          for k in c_mod (i.id_competicio,j.id_genere) loop
            for l in c_millor_jug (i.id_competicio, j.id_genere, k.id_modalitat) loop
              insert into millor_jugador values (j.id_genere,k.id_modalitat,l.jugador,l.vp);
            end loop;
          end loop;
        end loop;
      end loop;
    end if;
    -- Confirma les insercions a la taula de fets MILLOR_JUGADOR.
    commit;
		 -- Inserta a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
     -- nicament insereix el log si el parmetre LOG_ACTIU t el valor "S".
		 pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end prc_millor_jugador;


	-----------------
	-- PRC_ACT_DWH --
	-----------------
  procedure prc_act_dwh (
    p_data_ini date,
    p_data_fi date
  ) is

	-- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;

  begin

    -- Inicialitza la dimensi DIM_DATA amb el rang de dates introduit per parmetre.
    prc_ini_dim_data (p_data_ini, p_data_fi);

    -- Carrega les dades a la dimensi DIM_JUGADOR.
    prc_ini_dim_jugador (p_data_ini, p_data_fi);

    -- Carrega les dades a la dimensi DIM_EQUIP.
    prc_ini_dim_equip (p_data_ini, p_data_fi);

    -- Carrega les dades a la dimensi DIM_COMPETICIO.
    prc_ini_dim_competicio (p_data_ini, p_data_fi);

    -- Carrega les dades a la dimensi DIM_MODALITAT.
    prc_ini_dim_modalitat (p_data_ini, p_data_fi);

    -- Carrega les dades a la dimensi DIM_GENERE.
    prc_ini_dim_genere (p_data_ini, p_data_fi);

    -- Carrega les dades a la dimensi DIM_PAIS.
    prc_ini_dim_pais (p_data_ini, p_data_fi);

    -- Insereix a la taula de fets JUGADORS_ACTIUS un registre amb el id del gnere, el id de la modalitat
    -- i el nombre de jugadors actius d'aquests gnere i modalitat.
    prc_jug_actius (p_data_ini, p_data_fi);

	  -- Insereix a la taula de fets MAXIM_ANOTADOR un registre amb el id de la competicio, el id de la data,
    -- el id del jugador i el nmero de punts encistellats del jugador d'aquesta competici i data.
    prc_max_anotador (p_data_ini, p_data_fi);

    -- Insereix a la taula de fets JUGADOR_MES_BEN_PAGAT un registre amb el id de modalitat, el id del gnere,
    -- el id del jugador i el sou del jugador en aquesta modalitat i gnere.
    prc_jug_mes_ben_pagat (p_data_ini, p_data_fi);

    -- Insereix a la taula de fets JUGADOR_MES_GUANYS el id del jugador i els seus guanys.
    prc_jug_mes_guanys (p_data_ini, p_data_fi);

	  -- Insereix a la taula de fets RANKING_SOU_MIG el id del pas, el id de la data, el id del gnere, el id de la modalitat
    -- i el sou mig d'aquest pas, data, gnere i modalitat.
    prc_sou_mig (p_data_ini, p_data_fi);

    -- Insereix la taula de fets MILLORS_EQUIPS el id de la competici, el id de la data, el id de l'equip
    -- i els punts encistellas i rebuts de l'equip en aquesta competici i data.
    prc_millors_equips (p_data_ini, p_data_fi);

    -- Insereix a la taula de fets MILLOR_JUGADOR el id del gnere, el id de la modalitat, el id del jugador i la valoraci ponderada del jugador
    -- en aquest gnere i modalitat el darreny any.
    prc_millor_jugador (p_data_ini, p_data_fi);

	end prc_act_dwh;


  --------------------
  -- RES_JUG_ACTIUS --
  --------------------
  procedure res_jug_actius is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

  cursor c_jug_actius is
    select g.descripcio genere, m.descripcio modalitat, ja.num_jugadors
      from jugadors_actius ja, generes g, modalitats m
     where ja.id_genere = g.id
       and ja.id_modalitat = m.id
     order by g.descripcio, m.descripcio;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.RES_JUG_ACTIUS','N/A');

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		end if;

    -- Augmentem el tamany del buffer de l'output suficient perqu cpiga tot el missatge.
    dbms_output.enable(1000000);

    dbms_output.put_line('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.put_line('QUANTITAT DE JUGADORS EN ACTIU AGRUPATS PER GNERES I MODALITATS');
    dbms_output.put_line('+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
    dbms_output.new_line;

    for i in c_jug_actius loop
      dbms_output.put_line('GNERE: '||i.genere);
      dbms_output.put_line('MODALITAT: '||i.modalitat);
      dbms_output.put_line('NMERO DE JUGADORS ACTIUS: '||i.num_jugadors);
      dbms_output.new_line;
    end loop;
    -- Inserta a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end res_jug_actius;


  ------------------------
  -- RES_MAXIM_ANOTADOR --
  ------------------------
  procedure res_maxim_anotador (
    p_competicio_id varchar2
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

  cursor c_max_anotador is
    select distinct c.nom competicio, c.temporada, j.nom jugador, ma.punts
      from maxim_anotador ma, competicions c, jugadors j
     where ma.id_competicio = p_competicio_id
       and ma.id_competicio = c.id
       and ma.id_jugador = j.id;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.RES_MAXIM_ANOTADOR','PAR1 :'||p_competicio_id);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		end if;

    -- Augmentem el tamany del buffer de l'output suficient perqu cpiga tot el missatge.
    dbms_output.enable(1000000);
    for i in c_max_anotador loop
      dbms_output.put_line('++++++++++++++++++++++++++++');
      dbms_output.put_line('COMPETICI : '||i.competicio);
      dbms_output.put_line('TEMPORADA: '||i.temporada);
      dbms_output.put_line('++++++++++++++++++++++++++++');
      dbms_output.new_line;
      dbms_output.put_line('MXIM ANOTADOR: '||i.jugador);
      dbms_output.put_line('PUNTS: '||i.punts);
      dbms_output.new_line;
    end loop;
    -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end res_maxim_anotador;


  ---------------------------
  -- RES_JUG_MES_BEN_PAGAT --
  ---------------------------
  procedure res_jug_mes_ben_pagat is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

  cursor c_jug_pagat is
    select m.descripcio modalitat, g.descripcio genere, j.nom jugador, jp.sou
      from jugador_mes_ben_pagat jp, modalitats m, generes g, jugadors j
     where jp.id_modalitat = m.id
       and jp.id_genere = g.id
       and jp.id_jugador = j.id
     order by m.descripcio, g.descripcio;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.RES_JUG_MES_BEN_PAGAT','N/A');

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		end if;

    -- Augmentem el tamany del buffer de l'output suficient perqu cpiga tot el missatge.
    dbms_output.enable(1000000);
    for i in c_jug_pagat loop
      dbms_output.put_line('++++++++++++++++++++++++++++');
      dbms_output.put_line('MODALITAT : '||i.modalitat);
      dbms_output.put_line('GNERE: '||i.genere);
      dbms_output.put_line('++++++++++++++++++++++++++++');
      dbms_output.new_line;
      dbms_output.put_line('JUGADOR MS BEN PAGAT: '||i.jugador);
      dbms_output.put_line('SOU: '||to_char(i.sou,'999G999G990D00'));
      dbms_output.new_line;
    end loop;
    -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end res_jug_mes_ben_pagat;


  ------------------------
  -- RES_JUG_MES_GUANYS --
  ------------------------
  procedure res_jug_mes_guanys is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

  cursor c_jug_guanys is
    select distinct j.nom jugador, jm.guanys
      from jugador_mes_guany jm, jugadors j
     where jm.id_jugador = j.id;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.RES_JUG_MES_GUANYS','N/A');

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		end if;

    -- Augmentem el tamany del buffer de l'output suficient perqu cpiga tot el missatge.
    dbms_output.enable(1000000);
    for i in c_jug_guanys loop
      dbms_output.put_line('+++++++++++++++++++++++++++++++++++++');
      dbms_output.put_line('JUGADOR AMB MS GUANYS DE LA HISTRIA');
      dbms_output.put_line('+++++++++++++++++++++++++++++++++++++');
      dbms_output.new_line;
      dbms_output.put_line('JUGADOR: '||i.jugador);
      dbms_output.put_line('SOU: '||to_char(i.guanys,'999G999G990D00'));
      dbms_output.new_line;
    end loop;
    -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end res_jug_mes_guanys;

  -----------------
  -- RES_SOU_MIG --
  -----------------
  procedure res_sou_mig (
    p_pais varchar2,
    p_temporada varchar2,
    p_genere varchar2,
    p_modalitat varchar2
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

  cursor c_sou_mig is
    select distinct p.descripcio pais, c.temporada, g.descripcio genere, m.descripcio modalitat, r.sou_mig
      from ranking_sou_mig r, competicions c, paisos_continents p, generes g, modalitats m
     where r.id_pais = p_pais
       and r.id_genere = p_genere
       and r.id_modalitat = p_modalitat
       and c.temporada = p_temporada
       and r.id_pais = p.id
       and r.id_genere = g.id
       and r.id_modalitat = m.id;

	begin
		-- Regitra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.RES_SOU_MIG','PAR1: '||p_pais||
                                                        ' - PAR2: '||p_temporada||
                                                        ' - PAR3: '||p_genere||
                                                        ' - PAR4: '||p_modalitat);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		end if;

    -- Augmentem el tamany del buffer de l'output suficient perqu cpiga tot el missatge.
    dbms_output.enable(1000000);
    for i in c_sou_mig loop
      dbms_output.put_line('++++++++++++++++++++++++++++');
      dbms_output.put_line('PAS: '||i.pais);
      dbms_output.put_line('TEMPORADA: '||i.temporada);
      dbms_output.put_line('GNERE: '||i.genere);
      dbms_output.put_line('MODALITAT: '||i.modalitat);
      dbms_output.put_line('++++++++++++++++++++++++++++');
      dbms_output.new_line;
      dbms_output.put_line('SOU MIG: '||to_char(i.sou_mig,'999G999G990D00'));
      dbms_output.new_line;
    end loop;
    -- Insereix a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end res_sou_mig;


  ------------------------
  -- RES_MILLORS_EQUIPS --
  ------------------------
  procedure res_millors_equips (
    p_competicio_id number,
    p_temporada varchar2
  ) is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

  cursor c_mes_ofen is
    select distinct c.nom competicio, c.temporada, e.nom equip, me.punts_marcats
      from millors_equips me, competicions c, equips e
     where me.id_competicio = p_competicio_id
       and me.id_equip = e.id
       and me.punts_rebuts is null;

  r_mes_ofen c_mes_ofen%rowtype;

  cursor c_mes_defen is
    select distinct e.nom equip, me.punts_rebuts
      from millors_equips me, competicions c, equips e
     where me.id_competicio = p_competicio_id
       and me.id_equip = e.id
       and me.punts_marcats is null;

  r_mes_defen c_mes_defen%rowtype;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.RES_MILLORS_EQUIPS','PAR1: '||p_competicio_id||
                                                               ' - PAR2: '||p_temporada);

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		end if;

    -- Augmentem el tamany del buffer de l'output suficient perqu cpiga tot el missatge.
    dbms_output.enable(1000000);
    open c_mes_ofen;
      fetch c_mes_ofen into r_mes_ofen;
    close c_mes_ofen;
    open c_mes_defen;
      fetch c_mes_defen into r_mes_defen;
    close c_mes_defen;
    dbms_output.put_line('++++++++++++++++++++++++++++');
    dbms_output.put_line('COMPETICI: '||r_mes_ofen.competicio);
    dbms_output.put_line('TEMPORADA: '||r_mes_ofen.temporada);
    dbms_output.put_line('++++++++++++++++++++++++++++');
    dbms_output.new_line;
    dbms_output.put_line('EQUIP MS OFENSIU: '||r_mes_ofen.equip);
    dbms_output.put_line('EQUIP MS OFENSIU: '||r_mes_ofen.punts_marcats);
    dbms_output.new_line;
    dbms_output.put_line('EQUIP MS DEFENSIU: '||r_mes_defen.equip);
    dbms_output.put_line('EQUIP MS DEFENSIU: '||r_mes_defen.punts_rebuts);
    dbms_output.new_line;
    -- Inserta a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end res_millors_equips;


  ------------------------
  -- RES_MILLOR_JUGADOR --
  ------------------------
  procedure res_millor_jugador is

  -- Variable que s'utilitza per emmagatzemar el segent nmero de la seqncia SEQ_LOG_PROC_EXEC que ser l'identificador del LOG que es registra.
  v_id number;
	-- Variable que s'utilitza per emmagatzemar el valor del parmetre de LOG_ACTIU que indica si el sistema t el LOG activat o no.
	v_log_actiu boolean;

  cursor c_millor_jug is
    select distinct g.descripcio genere, m.descripcio modalitat, j.nom jugador, mj.vp
      from millor_jugador mj, generes g, modalitats m, jugadors j
     where mj.id_genere = g.id
       and mj.id_modalitat = m.id
       and mj.id_jugador = j.id
     order by g.descripcio, m.descripcio;

	begin
		-- Registra l'execuci del procediment amb els parmetres que s'ha executat
    -- i emmagatzema a la variable v_id l'identificador del log que retorna la funci FUN_REG_EXEC.
	  v_id:=pkg_control.fun_reg_exec(pck_nom||'.RES_MILLOR_JUGADOR','N/A');

		if v_test_actiu then
      -- Si el sistema es troba en mode TEST, s'insereix la paraula "TEST" al camp OPERACIO de la taula LOG_PROC_HIST_EXEC
      -- i es continua amb l'execuci del procediment normal.
		  pkg_control.prc_exec_test(v_id);
		end if;

    -- Augementem el tamany del buffer de l'output suficient perqu cpiga tot el missatge
    dbms_output.enable(1000000);
    for i in c_millor_jug loop
      dbms_output.put_line('+++++++++++++++++++++++++++++++++++++');
      dbms_output.put_line('MILLORS JUGADORS');
      dbms_output.put_line('+++++++++++++++++++++++++++++++++++++');
      dbms_output.new_line;
      dbms_output.put_line('GNERE: '||i.genere);
      dbms_output.put_line('MODALITAT: '||i.modalitat);
      dbms_output.put_line('JUGADOR: '||i.jugador);
      dbms_output.put_line('VALORACI PONDERADA: '||i.vp);
      dbms_output.new_line;
    end loop;
    -- Inserta a la taula LOG_PROC_HIST_EXEC el id del log, la data i hora actual i el text "FI" al camp OPERACIO.
    -- nicament inserta el log si el parmetre LOG_ACTIU t el valor "S".
		pkg_control.prc_exec_fi(v_id);
	 exception
	   -- Si l'execuci del procediment provoca una excepci es registra a la taula LOG_PROC_HIST_EXEC amb el text "ERROR: " seguit del detall del error.
     -- nicament inserta el log amb l'error si el parmetre LOG_ACTIU t el valor "S".
     when others then
	     pkg_control.prc_exec_error(v_id,'ERROR: '||substr(sqlerrm,1,500));

  end res_millor_jugador;

end pkg_dwh;
/

spool off;
